#! /usr/bin/env python3
import sys
import os
import subprocess
import argparse
################################################################################
# Compile script for Manyo-lib release version
#    ver 20240710
#    written by Y. Inamura
#########################################################################
"""
Usage : python3 do_make_Utsusemi.py <command> [options]
       command: [depend/man/make/install/clean]
       options: --debug --help --manyo-path
                --other-modules --numth --pyver
                --rev --without-depend --with-para --without-para
  -------------------------------------------------------
  depend  : depend script
  man     : script to make manual
  make    : make
  install : make install
  clean   : make clean

  --help           : show this help

  --manyo-path=/hoge/hogege/hoge
                   : set MANYO_PATH

  --dist-name=Ubuntu
                   : Linux distribution name [RH6,RH7,Ubuntu]
  --other-modules=SAS,DNA
                   : required other modules if need(default Utsusemi)

  --numth=n        : set the number of multi-threading
  --pyver=n.m      : set python version
                     If not set pyver, the version of python executing this is used
  --rev=nnnn       : set revision number if need
  --without-depend : does not run depend script
  --with-para      : use compile option of -j (default is no-use of -j)
  --without-para   : not use compile option of -j

  --debug          : debug mode
"""

#########################################################################
#   CXX            : compiler
#   PYVER          : version of python
#   NUM_OF_THREADS : number of threading
#   MANYO_PATH     : path to  manyo-core installed
#   INSTALL_PATH   : path to install
#   OTHER_MANYO_PAC: Other Manyo Package names required by compile
#   ENV_FILE_NAME  : file name of shell script to set Manyo environment
#   EXT_LIB_ROOT   : root path to external library required by Manyo-lib
#########################################################################
CXX = "g++"
PYVER = ""
try:
    PYVER = "{:d}.{:d}".format(sys.version_info.major, sys.version_info.minor)
except:
    pass
NUM_OF_THREADS = 8
MANYO_PATH = "/opt/mlfsoft/manyo"
INSTALL_PATH = MANYO_PATH
OTHER_MANYO_PAC = ["Utsusemi", "levmar"]
ENV_FILE_NAME = "bashrc.manyo.release"
EXT_LIB_ROOT = "/usr/local"
DIST_NAME = ""
if os.path.exists("/etc/redhat-release"):
    f = open("/etc/redhat-release", "r")
    r = f.readline()
    f.close()
    r_list = r.split(" ")
    for i, rel in enumerate(r_list):
        if rel == "release":
            p = r_list[i + 1]
            break
    if p.find("7") == 0 or p.find("8") == 0:
        DIST_NAME = "RH7"
    else:
        DIST_NAME = "RH6"
elif os.path.exists("/etc/os-release"):
    DIST_NAME = "Ubuntu"
else:
    pass

isDebug = False
isWithDepend = True
isCompPara = False
isNotManyoCore = True
utsusemi_rev = 0
if "UTSUSEMI_REV" in os.environ:
    utsusemi_rev = int(os.environ["UTSUSEMI_REV"])
else:
    try:
        cp = subprocess.run("git log -1 --date=short|grep 'Date:'", shell=True, capture_output=True, encoding="utf-8")
        if cp.stderr == '':
            latest_date_git = cp.stdout.split()[1]
            latest_date_v = latest_date_git.split("-")
            utsusemi_rev = latest_date_v[0][-2:] + latest_date_v[1] + latest_date_v[2]
    except:
        pass

#############################################
# Check arguments
#############################################

do_command = ""
parser = argparse.ArgumentParser(description="The script to compile manyo library source codes")

parser.add_argument('command', choices=['depend', 'man', 'make', 'install', 'clean'],
                    help='command in [depend/man/make/install/clean]')
parser.add_argument('--manyo-path', required=True, help='set MANYO_PATH')
parser.add_argument('--dist-name',
                    choices=["RH6", "RH7", "Ubuntu", "Ubuntu16", "Ubuntu18", "Ubuntu20", "Ubuntu22", "Ubuntu24"],
                    help="Disribution name")
parser.add_argument('--other-modules', choices=["SAS", "DNA"],
                    help="Other modules required if need (default Utsusemi")
parser.add_argument('--numth', type=int, help='set the number of multi-threading', default=2)
tp = lambda x:list(map(int, x.split('.')))
parser.add_argument('--pyver', type=tp, help='Python version')
parser.add_argument('--rev', help='revision number default:YYYYMMDD')
parser.add_argument('--with-para', action='store_true', help='do make command with option -j')
parser.add_argument('--without-para', action='store_true', help='do make command without option -j')
parser.add_argument('--without-depend', action='store_false', help='Not do depend script')
parser.add_argument('--debug', action='store_true', help='turn on the debug mode')

args = parser.parse_args()

do_command = args.command
MANYO_PATH = args.manyo_path
isDebug = args.debug
isWithDepend = args.without_depend
if args.with_para and (not args.without_para):
    isCompPara = True
else:
    isCompPara = False
if args.rev is not None:
    utsusemi_rev = args.rev
if args.pyver is not None:
    PYVER = "{}.{}".format(args.pyver[0], args.pyver[1])
if args.numth is not None:
    NUM_OF_THREADS = args.numth
if args.other_modules is not None:
    other_mods = args.other_modules.split(",")
    OTHER_MANYO_PAC.extend(other_mods)
    for a_mod in OTHER_MANYO_PAC:
        print("OTHER_MANYO_PAC= {}".format(a_mod))
if args.dist_name is not None:
    DIST_NAME = args.dist_name

if (do_command == "clean"):
    isWithDepend = False

if (do_command == "make"):
    do_command = ""

if os.path.exists(os.path.join(os.getcwd(), "core")):
    isNotManyoCore = False

#############################################
# Check compile target ( manyo-core or not )
#############################################
if isNotManyoCore:
    PACKAGE_NAME = ""
    if os.path.exists(os.path.join(os.getcwd(), "Makefile")):
        fo = open(os.path.join(os.getcwd(), "Makefile"), "r")
        ll = fo.readlines()
        fo.close()
        for aline in ll:
            print(aline)
            if aline.find("PACKAGE_NAME") != -1:
                PACKAGE_NAME = aline.split("=")[1].strip()
                break
    if PACKAGE_NAME == "":
        print("Makefile is invalid. (No PACKAGE_NAME)")
        sys.exit(1)

if DIST_NAME == "Ubuntu":
    try:
        fo = open("/etc/issue", "r")
        t = fo.readline()
        fo.close()
        v = t.split(' ')
        if v[1].find("24.04") == 0 or v[1].find("22.04") == 0 or v[1].find("20.04") == 0 or v[1].find("19.") == 0:
            DIST_NAME = "Ubuntu20"
        elif v[1].find("18.04") == 0 or v[1].find("17.") == 0 or v[1].find("16.04") == 0 or v[1].find("15.") == 0:
            DIST_NAME = "Ubuntu16"
    except:
        print(" ERROR : failed to distinguish version of Ubuntu")
        sys.exit(-1)
elif DIST_NAME == "Ubuntu18":
    DIST_NAME = "Ubuntu16"
elif DIST_NAME == "Ubuntu22" or DIST_NAME == "Ubuntu24":
    DIST_NAME = "Ubuntu20"


#########################################################################
# execute make or make install
#############################################
OTHER_MAN_PAC = ["Manyo", "MLF"]
for pac in OTHER_MANYO_PAC:
    OTHER_MAN_PAC.append(pac)

EXT_LIB_PATH = os.path.join(EXT_LIB_ROOT, "lib")
EXT_INC_PATH = os.path.join(EXT_LIB_ROOT, "include")

com = "export MANYO_PATH={:s};".format(MANYO_PATH)
if isNotManyoCore:
    com += "export INSTALL_PATH={:s};".format(INSTALL_PATH)
    com += "export LIBRARY_PATH=${MANYO_PATH}/lib:${INSTALL_PATH}:"
    com += "{:s};".format(EXT_LIB_PATH)
else:
    com += "export LIBRARY_PATH=${MANYO_PATH}/lib:"
    com += "{:s};".format(EXT_LIB_PATH)
com += "export LD_LIBRARY_PATH=${LIBRARY_PATH};"
if PYVER != "":
    if DIST_NAME.find("Ubuntu20") == 0:
        com += "export C_INCLUDE_PATH=/usr/include/nexus:/usr/include/hdf5/serial:${MANYO_PATH}/include:"
        com += "/usr/include/python{:s}:{:s}".format(PYVER, EXT_INC_PATH)  # [inamura 201103 for Ubuntu 20.04]
    elif DIST_NAME.find("Ubuntu16") == 0:  # for Ubuntu 16 or 18
        com += "export C_INCLUDE_PATH=/usr/include/hdf5/serial:${MANYO_PATH}/include:"
        com += "/usr/include/python{:s}:{:s}".format(PYVER, EXT_INC_PATH)  # [inamura 201103 for Ubuntu 16.04 and Ubuntu 18.04]
    else:
        com += "export C_INCLUDE_PATH=${MANYO_PATH}/include:"
        com += "{:s}".format(EXT_INC_PATH)
else:
    com += "export C_INCLUDE_PATH=/usr/include/hdf5/serial:${MANYO_PATH}/include:"
    com += "{:s}".format(EXT_INC_PATH)
for m_pac_name in OTHER_MAN_PAC:
    com += ":${MANYO_PATH}/include/"
    com += "{:s}".format(m_pac_name)
com += "; "
com += 'export INC_EXT="'
for m_pac_name in OTHER_MAN_PAC:
    com += "{:s} ".format(m_pac_name)
com += '"; '
com += "export CPLUS_INCLUDE_PATH=${C_INCLUDE_PATH};"
com += "export PYTHONPATH=${MANYO_PATH};"
com += "export PATH=${MANYO_PATH}/bin:${PATH};"
if isNotManyoCore and isWithDepend:
    com += "./depend; "

if (do_command != "depend"):
    com += 'make {:s} '.format(do_command)
    if isCompPara:
        com += '-j '
    if DIST_NAME == "":
        DIST_NAME = "Ubuntu"
    com += ' DIST_NAME={:s}'.format(DIST_NAME)
    if PYVER != "":
        com += ' CXX={:s} PyVer={:s} NUM_MULTH={:d}'.format(CXX, PYVER, NUM_OF_THREADS)
    else:
        com += ' CXX={:s} NUM_MULTH={:d}'.format(CXX, NUM_OF_THREADS)
    com += ' UTSUSEMI_REV={}'.format(utsusemi_rev)

print("command = {}".format(com))
if isDebug:
    pass
else:
    retcode = os.system(com)
    if retcode:
        sys.stderr.write("Return code '{}', is not 0.\n".format(retcode))
        raise RuntimeError("Error has been caused in building.")


#########################################################################
# make environ file
#############################################

env_list = []
env_list.append('#!/bin/bash')
env_list.append('export MANYO_PATH={:s}'.format(MANYO_PATH))

if isNotManyoCore and (MANYO_PATH != INSTALL_PATH):
    env_list.append('export INSTALL_PATH={:s}'.format(INSTALL_PATH))
    envset = 'export LD_LIBRARY_PATH=${MANYO_PATH}/lib:${INSTALL_PATH}/'
    envset += '{:s}/lib:'.format(PACKAGE_NAME)
    envset += '${LD_LIBRARY_PATH}'
    env_list.append(envset)
    envset = 'export PYTHONPATH=${MANYO_PATH}:${INSTALL_PATH}/Manyo/'
    envset += '{:s}:'.format(PACKAGE_NAME)
    envset += '${PYTHONPATH}'
    env_list.append(envset)
else:
    env_list.append(
        'export LD_LIBRARY_PATH=${MANYO_PATH}/lib:${LD_LIBRARY_PATH}')
    env_list.append('export PYTHONPATH=${MANYO_PATH}:${PYTHONPATH}')
env_list.append('export PATH=${MANYO_PATH}/bin:$PATH')


if isDebug:
    for an_env in env_list:
        print(an_env)
else:
    fo = open(os.path.join(os.getcwd(), ENV_FILE_NAME), 'w')
    for ss in env_list:
        fo.write(ss + '\n')
    fo.close()



from __future__ import print_function
import os,sys,time
ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
if ver<2.5:
    from elementtree.ElementTree import ElementTree,fromstring,XML,tostring
else:
    from xml.etree.ElementTree import ElementTree,fromstring,XML,tostring

from numpy import *
import time

import Manyo as mm
import six
#import vis.VisualContCommon as VCC

################################
#      VisContQ parameter
################################
class VisContQParams(object):
    """
    Treating parameter file for VisContQ seriese
            [inamura 160619] Bugfix of treating SampleRotateSteps
    ver 0.3 [Inamura 150527] add params for rotation steps
    ver 0.2 [Inamura 130714] add params for diagonal folding
    ver 0.1 [Inamura 110702]
    
    """
    __version__="0.3"

    #####################################
    def __init__(self,filepath=""):
        """
        Constructor
        @param filepath (string) path/to/xmlfile
        @return None
        """
        
        self.filepath = filepath
        self.root_ele = None        
        self.Header  = {}
        
        self.paxis_info = None
        self.proj_axes = None
        
        self.type_dic={"x":0,"y":0,"p":0,"t":0}
        
        self.diag_folding = [0,0,0]  ##[inamura 130714]
        
        self.EnableVisType=None

        self.CommentHead = "VisContQParams >> "

        if self.filepath!="":
            try:
                self.SetParamFile( self.filepath )
                self.LoadVisualContParams()
            except:
                print(self.CommentHead+"::constructor >> False to LoadVisualContParams")
                pass

    #####################################
    def SetParamFile(self, filepath=""):
        """
        Set file to be imported parameters.
        @param filepath (String) path to xml file.
        @return None
        """
        if filepath!="":
            self.filepath = filepath
        
        if os.path.exists(self.filepath):
            fd = open(self.filepath,'rb')
            dom = ElementTree(file=fd)
            self.root_ele = dom.getroot()
            fd.close()
        else:
            print(self.CommentHead + "Cannot read file. :"+self.filepath)
            raise

    #####################################
    def LoadVisualContParams(self):
        """
        Import parameters to handle VisCont.
        @params None
        @return None
        """
        if self.root_ele == None:
            print(self.CommentHead + "You did not import file.")
            return
        
        # import the information of lattice constants from XML file
        ##self.smpInfo = VCC.SampleInfo(None)
        
        smpInfo_ele = self.root_ele.find("sampleInformation")
        sliceInfo_ele = self.root_ele.find("slicingInformation")

        samplename_text = smpInfo_ele.find("name").text
        lattice_ele = smpInfo_ele.find("latticeInfo")
        lconst_text = lattice_ele.find("latticeConst").text
        langle_text = lattice_ele.find("latticeAngle").text
        orient_ele = smpInfo_ele.find("orientation")
        Uvec_text = orient_ele.find("Uvector").text
        Vvec_text = orient_ele.find("Vvector").text
        phi_ele = orient_ele.find("phi")
        ##[inamura 150527]-->
        phi_text = "0.0"
        try:
            if phi_ele==None:
                phi_text = orient_ele.find("fai").text
                print("VisContQParam >> This param file is old version.")
            else:
                phi_text = phi_ele.text
        except:
            pass
        
        rotstep_list = []
        rotstep_ele = orient_ele.find("rotateSteps")
        if rotstep_ele!=None:
            #[inamura 160619]-->
            if isinstance(rotstep_ele.text,str):
                rotstep_text = rotstep_ele.text 
                rotstep_text_list = rotstep_text.split(",")
                for a_step in rotstep_text_list:
                    steps = a_step.split(":")
                    if len(steps)!=2:
                        print("VisContQParam >> This rotation steps info is invalid.")
                        return
                    rotstep_list.append( [ steps[0], float(steps[1]) ] )
            #<--[inamura 160619]
        if (float(phi_text)!=0.0) and (len(rotstep_list)==0):
            rotstep_list.append( ["Y", float(phi_text)] )
        ##<--[inamura 150527]
            
        LC = []
        for tmp in lconst_text.split(","):
            LC.append(float(tmp))
        
        LA = []
        for tmp in langle_text.split(","):
            LA.append(float(tmp))
        
        UV = []
        for tmp in Uvec_text.split(","):
            UV.append(float(tmp))
            
        VV = []
        for tmp in Vvec_text.split(","):
            VV.append(float(tmp))

        self.Header={}

        self.Header['SampleName'] = samplename_text
        self.Header['SimpleLatticeConsts']=LC+LA
        self.Header['SampleUvec']=UV
        self.Header['SampleVvec']=VV
        #self.Header['SampleRotatePhi']=float(phi_text)
        self.Header['SampleRotateSteps']= rotstep_list         ##[inamura 150527]
        
        """
        self.smpInfo.a = float(LC[0])
        self.smpInfo.b = float(LC[1])
        self.smpInfo.c = float(LC[2])
        self.smpInfo.alpha = float(LA[0])
        self.smpInfo.beta  = float(LA[1])
        self.smpInfo.gamma = float(LA[2])
        self.smpInfo.uh = float(UV[0])
        self.smpInfo.uk = float(UV[1])
        self.smpInfo.ul = float(UV[2])
        self.smpInfo.vh = float(VV[0])
        self.smpInfo.vk = float(VV[1])
        self.smpInfo.vl = float(VV[2])
        
        self.smpInfo.fai = float(fai_text)
        """
        
        # import parameters for the projection
        self.proj_axes = {}
        project_ele = sliceInfo_ele.find("projectParams")
        axis_ele_list = project_ele.findall("axis")
        
        for item in axis_ele_list:
            id = item.get("id")
            vec_el = item.find("vector")
            lt = vec_el.text.split(",")
            title = item.find("title").text
            self.proj_axes[id]=[ float(lt[0]),float(lt[1]),float(lt[2]),float(lt[3]),title ]

        # import parameters for slicing using projection parameters
        plot_ele = sliceInfo_ele.find("plotParams")
        paxis_ele_list = plot_ele.findall("paxis")
        
        self.paxis_info = {}
        
        for ele in paxis_ele_list:
            id_str = ele.get("id")
            #print "id=",id_str
            type_str = ele.find("type").text
            if type_str not in self.type_dic:
                print(self.CommentHead + "type is invalid at id= {}".format(id_str))
                raise
            else:
                self.type_dic[ type_str ]+=1

            range_axis = []
            if type_str!="p":
                range_str = ele.find("range").text
                range_list = range_str.split(",")
                for item in range_list:
                    range_axis.append( float(item) )
                if len(range_list)==2:
                    range_axis.append( range_axis[1]-range_axis[0] )

            folding_str = ele.find("folding").text
            ##[inamura 130624]-->
            #if (folding_str==None) or (not (folding_str in ["0","1"]) ):
            #    folding=0
            #else:
            #    folding=int(folding_str)
            folding = -1.0
            if (folding_str!=None):
                folding = float( folding_str )
                if folding<0:
                    folding = -1.0
            ##<--[inamura130624]
            
            self.paxis_info[id_str] = ( type_str, range_axis, folding )

        if (self.type_dic["x"]==1) and (self.type_dic["y"]==1):
            if (self.type_dic["p"]==1) and (self.type_dic["t"]==1):
                self.EnableVisType="VisualCont"
            elif (self.type_dic["p"]==0) and (self.type_dic["t"]==2):
                self.EnableVisType="VisualCont2"

        ##[inamura 130714]-->
        dfold = plot_ele.find("diagFolding")
        if dfold!=None:
            dfold_type=int( dfold.find("type").text )
            if dfold_type in [1,2]:
                self.diag_folding[0] = dfold_type
                dfold_axes_st= dfold.find("usedAxis").text
                dfold_axes = dfold_axes_st.split(",")
                ind = 1
                for fold_axis in dfold_axes:
                    if fold_axis in ["1","2","3","4"]:
                        self.diag_folding[ind]=int(fold_axis)
                        ind +=1
                    else:
                        self.diag_folding[1]=0
                        self.diag_folding[2]=0
                        break
            else:
                self.diag_folding[0] = 0
        ##<--[inamura 130714]
    #####################################
    def SetLatticeConst(self,LC=[],LA=[]):
        """
        Set Lattice constant
        @param LC (list of float) Lattice constants [Angstrome]
        @[aram LA (list of float) Lattice angles [degree]
        @return None
        """
        if self.root_ele == None:
            print(self.CommentHead + "You did not import file.")
            raise

        smpInfo_ele = self.root_ele.find("sampleInformation")
        lattice_ele = smpInfo_ele.find("latticeInfo")
        
        if 'SimpleLatticeConsts' in self.Header:
            Alc = self.Header['SimpleLatticeConsts']
        else:
            Alc = [0.0,0.0,0.0,0.0,0.0,0.0]
            
        if len(LC)==3:
            Alc[0] = float(LC[0])
            Alc[1] = float(LC[1])
            Alc[2] = float(LC[2])
            lattice_ele.find("latticeConst").text = "%f, %f, %f" % (Alc[0],Alc[1],Alc[2])
        if len(LA)==3:
            Alc[3] = float(LA[0])
            Alc[4] = float(LA[1])
            Alc[5] = float(LA[2])            
            lattice_ele.find("latticeAngle").text = "%f, %f, %f" % (Alc[3],Alc[4],Alc[5])
        
        self.Header['SimpleLatticeConsts'] = Alc
        
    #####################################
    def SetOrientationVect(self,Uvec=[],Vvec=[]):
        """
        Set Sample Orientation information
        @param Uvec (list of float) U vector (parallel to ki)
        @param Vvec (list of float) V vector (perpendicular to ki)
        @return None
        """
        if self.root_ele == None:
            print(self.CommentHead + "You did not import file.")
            raise

        smpInfo_ele = self.root_ele.find("sampleInformation")
        orient_ele = smpInfo_ele.find("orientation")
        
        if "SampleUvec" in self.Header:
            h_Uv = self.Header["SampleUvec"]
            h_Vv = self.Header["SampleVvec"]
        else:
            h_Uv = [0.0,0.0,0.0]
            h_Vv = [0.0,0.0,0.0]
        
        if len(Uvec)==3:
            h_Uv[0] = float(Uvec[0])
            h_Uv[1] = float(Uvec[1])
            h_Uv[2] = float(Uvec[2])
            orient_ele.find("Uvector").text = "%f, %f, %f" % (h_Uv[0],h_Uv[1],h_Uv[2])
        if len(Vvec)==3:
            h_Vv[0] = float(Vvec[0])
            h_Vv[1] = float(Vvec[1])
            h_Vv[2] = float(Vvec[2])        
            orient_ele.find("Vvector").text = "%f, %f, %f" % (h_Vv[0], h_Vv[1], h_Vv[2])
        
        self.Header["SampleUvec"] = h_Uv
        self.Header["SampleVvec"] = h_Vv

    #####################################
    def SetOrientationPhi(self,phi=None):
        """
        Set Sample orientation angle
        @param fai (float) angle between U vector and ki vector [degree]
        @param return
        """
        if self.root_ele == None:
            print(self.CommentHead + "You did not import file.")
            raise
        
        if phi!=None:
            self.Header['SampleRotatePhi'] = float(phi)
            smpInfo_ele = self.root_ele.find("sampleInformation")
            orient_ele = smpInfo_ele.find("orientation")
            orient_ele.find("phi").text = "%f" % (self.Header['SampleRotatePhi'])
            rotateSteps_ele = orient_ele.find("rotateSteps")
            SRS_cont=[] #[inamura 160619]
            if rotateSteps_ele!=None:
                #[inamura 160619]-->
                if 'SampleRotateSteps' in self.Header:
                    RL = self.Header['SampleRotateSteps']
                    tmp_text=""
                    for a_RL in RL:
                        if a_RL[0]=="Y":
                            a_RL[1]=float(phi)
                        tmp_text+="%s:%f,"%(a_RL[0],a_RL[1])
                        SRS_cont.append( [a_RL[0],a_RL[1]] )
                    rotateSteps_ele.text = tmp_text[:-1]
                else:
                    SRS_cont = [ ["Y",float(phi)] ]
                
            self.Header['SampleRotateSteps'] = SRS_cont[:]
            #<--[inamura 160619]
            
    #####################################
    def SetOrientationRotationSteps(self,steps=[]):
        """
        Set steps of Sample orientation axis and its angle
        [inamura 150527]
        @param steps (list)  [degree]
        @param return
        """
        if self.root_ele == None:
            print(self.CommentHead + "You did not import file.")
            raise
        
        if len(steps)!=0:
            steps_text = ""
            for i in range( steps ):
                ang_info = steps[i]
                steps_text += "%s:%f,"%(steps[i][0],steps[i][1])
            steps_text = steps_text[:-1]
            
            smpInfo_ele = self.root_ele.find("sampleInformation")
            orient_ele = smpInfo_ele.find("orientation")
            orient_ele.find("rotateSteps").text = steps_text

    #####################################
    def SetProjectionInfo(self,terget_id=-1,vect=[],title=""):
        """
        Set Projection information
        @param terget_id (int) ID for given axis
        @param vect      (list of float) vector for axis
        @param title     (string) title of axis
        @return None
        """
        if self.root_ele == None:
            print(self.CommentHead + "You did not import file.")
            raise

        if not (target_id in [0,1,2,3]):
            print(self.CommentHead + "ID is invalid!!!")
            raise

        if len(vect)!=3:
            print(self.CommentHead + "Vector is invalid!!!")
            raise
        
        sliceInfo_ele = self.root_ele.find("slicingInformation")
        project_ele = sliceInfo_ele.find("projectParams")
        axis_ele_list = project_ele.findall("axis")
        for item in axis_ele_list:
            id = item.get("id")
            if int(id) == target_id:
                self.proj_axes[id]=[ vect[0], vect[1], vect[2], vect[3], title ]
                item.find("vector").text = "%f, %f, %f, %f" % ( vect[0],vect[1],vect[2],vect[3] )
                item.find("title").text = title

    #####################################
    def SetSliceInfo(self,target_id=-1,type="",axrange=[],folding=-1.0):
        """
        Set Slicing parameters
        @param target_id (int) ID for axis
        @param type      (string) "x":x-axis, "y":y-axis, "p":projection axis, "t":thickness
        @param axrange   (list)
        @param folding   (float) folding flag 
        @return None
        """
        if self.root_ele == None:
            msg = self.CommentHead + "You did not import file."
            print(msg)
            raise UserWarning(msg)
        if not (target_id in [0,1,2,3]):
            msg = self.CommentHead + "ID is invalid!!!"
            print(msg)
            raise UserWarning(msg)

        if not (type in self.type_dic):
            msg = self.CommentHead + "Type is invalid!!! ( Choose x, y, p, t )"
            print(msg)
            raise UserWarning(msg)
        
        if (len(axrange)!=2) and (len(axrange)!=3):
            msg = self.CommentHead + "Vector is invalid!!!"
            print(msg)
            raise UserWarning(msg)

        sliceInfo_ele = self.root_ele.find("slicingInformation")
        plot_ele = sliceInfo_ele.find("plotParams")
        paxis_ele_list = plot_ele.findall("paxis")

        
        for ele in paxis_ele_list:
            if int(ele.get("id"))==target_id:
                ele.find("type").text = type
                if (len(axrange)==2) and ( (type=="p") or (type=="t") ):
                    ele.find("range").text = "%f, %f, %f" % (axrange[0],axrange[1],(axrange[1]-axrange[0]))
                elif (len(axrange)==3) and ( (type=="p") or (type=="t") ):
                    ele.find("range").text = "%f, %f, %f" % (axrange[0],axrange[1],axrange[2])
                elif (len(axrange)==3) and ( (type=="x") or (type=="y") ):
                    ele.find("range").text = "%f, %f, %f" % (axrange[0],axrange[1],axrange[2])
                else:
                    print(self.CommentHead + "Type and range is not mutch.")
                    raise
                ##[inamura 130624]-->
                #if folding:
                #    fol = 1
                #else:
                #    fol = 0
                if folding<0:
                    fol = -1.0
                else:
                    fol = folding
                ##<--[inamura 130624]
                    
                self.paxis_info[str(target_id)] = ( type, axrange, fol )
                        
        self.type_dic={"x":0,"y":0,"p":0,"t":0}
        for ele in paxis_ele_list:
            type_str = ele.find("type").text
            self.type_dic[ type_str ] += 1

        
        self.EnableVisType=None
        if (self.type_dic["x"]==1) and (self.type_dic["y"]==1):
            if (self.type_dic["p"]==1) and (self.type_dic["t"]==1):
                self.EnableVisType="VisualCont"
            elif (self.type_dic["p"]==0) and (self.type_dic["t"]==2):
                self.EnableVisType="VisualCont2"

    #####################################
    def SetDiagFoldingInfo(self,fold_type,axes=[]):
        """
        Set Diagonal Folding setting info.
        @param fold_type (int) 0:not use, 1:first-third quadrant, 2:second-forth quadrant
        @param axes ( list of int ) 2 axes for diagonal folding
        """
        if not fold_type in [0,1,2]:
            msg = self.CommentHead + "1st argument is invalid!!! (fold_type=" + str(fold_type) + ")"
            print(msg)
            raise UserWarning(msg)
        
        if len(axes)!=2:
            msg = self.CommentHead + "2nd argument is invalid!!! (ftype=" + str(fold_type) + ")"
            print(msg)
            raise UserWarning(msg)
        
        if (axes[0] in [1,2,3,4]) and (axes[1] in [1,2,3,4]):
            self.diag_folding[0] = fold_type
            self.diag_folding[1] = axes[0]
            self.diag_folding[2] = axes[1]
        else:
            self.diag_folding[0] = 0
            self.diag_folding[1] = 0
            self.diag_folding[2] = 0
        
        sliceInfo_ele = self.root_ele.find("slicingInformation")
        plot_ele = sliceInfo_ele.find("plotParams")
        if plot_ele!=None:
            diag_ele = plot_ele.find("diagFolding")
            if diag_ele!=None:
                type_ele = diag_ele.find("type")
                type_ele.text = str(self.diag_folding[0])
                usedAxis_ele = diag_ele.find("usedAxis")
                usedAxis_ele.text = "%d,%d" % (self.diag_folding[1],self.diag_folding[2])
            
    #####################################            
    def MakeNewParams(self):
        """
        Import new format for parameter.
        @param None
        @return None
        """
        xml_st = self.GetParamFileTemplete()
        #self.root_ele = fromstring(xml_st)
        self.root_ele = XML(xml_st)
        self.LoadVisualContParams()

    ##[inamura 110205]-->
    #####################################
    def SetAllInfo(self,Header_in,axisInfo=[],sliceInfo=[],foldInfo=[]):
        """
        Header_in = {"SimpleLatticeConsts":[a,b,c,alpha,beta,gamma],
                     "SampleUvec":[uh,uk,ul],
                     "SampleVvec":[vh,vk,vl],
                     "SampleRotatePhi":phi }

        axisInfo= [ [ax1_a, ax1_b, ax1_c, ax1_hw, ax1_title(string)],
                    [ax2_a, ax2_b, ax2_c, ax2_hw, ax2_title(string)],
                    [ax3_a, ax3_b, ax3_c, ax3_hw, ax3_title(string)],
                    [ax4_a, ax4_b, ax4_c, ax4_hw, ax4_title(string)] ]
        sliceInfo=[ [type1, ax1_min, ax1_max, ax1_step, ax1_folding ],
                    ...
                    [type4, ax4_min, ax4_max, ax4_step, ax4_folding ] ]
        foldInfo= [ diagonal_folding_type, axis1, axis2 ]
        
        @param smpInfo_in (smpInfo class)
        @param axisInfo   (list of axis info)
        @param sliceInfo  (list of slicing parameter)
        @return None
        """
        self.Header = Header_in
        lc = self.Header["SimpleLatticeConsts"]
        uv = self.Header["SampleUvec"]
        vv = self.Header["SampleVvec"]
        
        self.root_ele = XML( self.GetParamFileTemplete() )
        smpInfo_ele = self.root_ele.find("sampleInformation")
        
        lattice_ele = smpInfo_ele.find("latticeInfo")
        lattice_ele.find("latticeConst").text ="%f, %f, %f"% (lc[0],lc[1],lc[2])
        lattice_ele.find("latticeAngle").text = "%f, %f, %f" % (lc[3],lc[4],lc[5])
        
        orient_ele = smpInfo_ele.find("orientation")
        orient_ele.find("Uvector").text = "%f, %f, %f" % (uv[0],uv[1],uv[2])
        orient_ele.find("Vvector").text = "%f, %f, %f" % (vv[0],vv[1],vv[2])
        ##[inamura 150527]-->
        #orient_ele.find("phi").text = "%f" % (self.Header["SampleRotatePhi"])
        rotstep_list = self.Header['SampleRotateSteps']
        steps_text = ""
        for i in range( len(rotstep_list) ):
            ang_info = rotstep_list[i]
            steps_text += "%s:%f,"%(rotstep_list[i][0],rotstep_list[i][1])
        steps_text = steps_text[:-1]
        orient_ele.find("rotateSteps").text = steps_text
        ##<--[inamura 150527]
        
        self.proj_axes = {}
        
        sliceInfo_ele = self.root_ele.find("slicingInformation")
        project_ele = sliceInfo_ele.find("projectParams")
        axis_ele_list = project_ele.findall("axis")

        for (i,axis) in enumerate(axis_ele_list):
            self.proj_axes[str(i)]=[axisInfo[i][0],axisInfo[i][1],axisInfo[i][2],axisInfo[i][3]
                                    ,axisInfo[i][4]]
            axis.find("vector").text = "%f, %f, %f, %f" % ( axisInfo[i][0],axisInfo[i][1],axisInfo[i][2],axisInfo[i][3] )
            axis.find("title").text = axisInfo[i][4]
        
        
        self.paxis_info = {}
        
        plot_ele = sliceInfo_ele.find("plotParams")
        paxis_ele_list = plot_ele.findall("paxis")
        for (i,paxis) in enumerate(paxis_ele_list):
            paxis.find("type").text = sliceInfo[i][0]
            if (sliceInfo[i][0]=="t") and ( len(sliceInfo[i])==2 ):
                axrange = [sliceInfo[i][1],sliceInfo[i][2]]
                paxis.find("range").text = "%f, %f, %f" % (axrange[0],axrange[1],(axrange[1]-axrange[0]))
            elif (sliceInfo[i][0] in ["x","y","t"]):
                axrange = [sliceInfo[i][1],sliceInfo[i][2],sliceInfo[i][3]]
                paxis.find("range").text = "%f, %f, %f" % (axrange[0],axrange[1],axrange[2])
            else:
                axrange = []
                
            paxis.find("folding").text = str(sliceInfo[i][4])

            self.paxis_info[str(i)] = ( sliceInfo[i][0], axrange, int(sliceInfo[i][4]) )

        self.type_dic={"x":0,"y":0,"p":0,"t":0}
        for ele in paxis_ele_list:
            type_str = ele.find("type").text
            self.type_dic[ type_str ] += 1

        
        self.EnableVisType=None
        if (self.type_dic["x"]==1) and (self.type_dic["y"]==1):
            if (self.type_dic["p"]==1) and (self.type_dic["t"]==1):
                self.EnableVisType="VisualCont"
            elif (self.type_dic["p"]==0) and (self.type_dic["t"]==2):
                self.EnableVisType="VisualCont2"
        
        if len(foldInfo)==3:
            self.SetDiagFoldingInfo(foldInfo[0],foldInfo[1:])
            
    ##<--[inamura 110205]
        
    #####################################
    def ShowParams(self):
        """
        Show all parameters
        @param None
        @return None
        """
        if self.root_ele == None:
            print(self.CommentHead + "You did not import file.")
            raise

        lc = self.Header["SimpleLatticeConsts"]
        uv = self.Header["SampleUvec"]
        vv = self.Header["SampleVvec"]

        print("Lattice constants")
        print("a = %f" % lc[0])
        print("b = %f" % lc[1])
        print("c = %f" % lc[2])
        print("alpha = %f" % lc[3])
        print("beta  = %f" % lc[4])
        print("gamma = %f" % lc[5])
        print("---")
        print("Sample orientation")
        print("Uvec = [ %f, %f, %f ]" % (uv[0],uv[1],uv[2]))
        print("Vvec = [ %f, %f, %f ]" % (vv[0],vv[1],vv[2]))
        ##[inamura 150527]-->
        #print "phi = %f" % self.Header['SampleRotatePhi']
        rotstep_list = self.Header['SampleRotateSteps']
        for i,ang in enumerate(rotstep_list):
            print("Rotation Step %d : axis=%s, and=%f"%(i,ang[0],ang[1]))
        ##<--[inamura 150527]
        print("---")
        print("Projection")
        proj_keys = self.proj_axes.keys()
        proj_keys.sort()
        for pkey in proj_keys:
            [lt0,lt1,lt2,lt3,title]=self.proj_axes[pkey]
            print("ID(%s): [%f,%f,%f,%f] :%s" % (pkey,lt0,lt1,lt2,lt3,title))

        print("---")
        print("Slicing")
        SS = ""
        paxis_ids = self.paxis_info.keys()
        paxis_ids.sort()
        for id in paxis_ids:
            (type_str, range_axis, folding) = self.paxis_info[id]
            SS = "ID(%s) : Type=%s : " % (id,type_str)
            for rr in range_axis:
                SS += " %f," % rr
            SS += " Folding=%f" % folding ##[inamura 130624]
            print(SS)

        print("---")
        print("Diagonal Folding")
        if self.diag_folding[0]==1:
            print("Folding along diagonal line through 1st-3rd quadrant (axes id=%d,%d)" % (self.diag_folding[1],self.diag_folding[2]))
        elif self.diag_folding[0]==2:
            print("Folding along diagonal line through 2nd-4th quadrant (axes id=%d,%d)" % (self.diag_folding[1],self.diag_folding[2]))
        else:
            print("NO SET")

    #####################################
    def ShowXML(self):
        """
        Show XML introduced from element
        @param None
        @return None
        """
        if six.PY2:
            print(tostring(self.root_ele))
        else:
            print(tostring(self.root_ele,"unicode"))

    #####################################
    def SaveXML(self,filename=""):
        """
        Save parameter to XML file
        @param filename (string) path and name of xml file to be saved
        @return None
        """
        if self.root_ele == None:
            print(self.CommentHead + "You did not import nor make parameters.")
            raise

        if os.path.exists(filename):
            print(self.CommentHead+filename+" is already existed.")
            
        try:
            fd = open( filename,"w" )
            fd.write( '<?xml version="1.0" encoding="utf-8"?>\n' )
            if six.PY2:
                fd.write( tostring(self.root_ele) )
            else:
                fd.write( tostring(self.root_ele,"unicode") )
            fd.close()
        except:
            print(self.CommentHead + " failed save to file.")
            raise
        
    #####################################            
    def GetParamFileTemplete(self):
        """
        Return the templete for XML file
        @param None
        @return string of XML
        """
        return """

<visualContParams version="0.3">
  <sampleInformation>
    <name>CuGeO3</name>
    <latticeInfo>
      <latticeConst>4.81, 8.47, 2.941</latticeConst>
      <latticeAngle>90.0, 90.0, 90.0</latticeAngle>
    </latticeInfo>
    <orientation>
      <Uvector>0.0, 1.0, 0.0</Uvector>
      <Vvector>0.0, 0.0, 1.0</Vvector>
      <phi>0.0</phi>
      <rotateSteps></rotateSteps>
    </orientation>
  </sampleInformation>
  <slicingInformation>
    <projectParams>
      <axis id="0">
        <vector>1.0, 0.0, 0.0, 0.0</vector>
        <title>Qa</title>
      </axis>
      <axis id="1">
        <vector>0.0, 1.0, 0.0, 0.0</vector>
        <title>Qb</title>
      </axis>
      <axis id="2">
        <vector>0.0, 0.0, 1.0, 0.0</vector>
        <title>Qc</title>
      </axis>
      <axis id="3">
        <vector>0.0, 0.0, 0.0, 1.0</vector>
        <title>hw</title>
      </axis>    
    </projectParams>

    <plotParams>
      <paxis id="0">
        <type>t</type>
        <range>-5.0,5.0,0.05</range>
        <folding>-1.0</folding>
      </paxis>
      <paxis id="1">
        <type>p</type>
        <range></range>
        <folding>-1.0</folding>
      </paxis>
      <paxis id="2">
        <type>x</type>
        <range>-1.5,2.0,0.02</range>
        <folding>-1.0</folding>
      </paxis>
      <paxis id="3">
        <type>y</type>
        <range>-5.0,40.0,0.5</range>
        <folding>-1.0</folding>
      </paxis>
      <diagFolding>
        <type>0</type>
        <usedAxis>0,0</usedAxis>
      </diagFolding>
    </plotParams>
  </slicingInformation>
</visualContParams>
"""
      



#! /usr/bin/env python
# -*- coding:utf-8 -*-
import tkinter as Tkinter
import datetime
import time
import os
import shutil
import Manyo as mm
import Manyo.Utsusemi as mu
import Cmm
import uGao.MPlot as MP
import uGao.M2PlotPlusCui as MP2
import ElasticScanGUIBase as ESB
import T0fileMake as T0M
import subprocess

import ElasticScanPy3 as ESP3

class App(Tkinter.Frame):
    def __init__(self, master = None, ):
        Tkinter.Frame.__init__(self, master,bg="#00FFFF")
        self.master.title("DNA Elastic Scan Reduction Check")
        self.errorstr = Tkinter.StringVar()
        self.errorstr.set("Elastic Scan Stand-by.")

        display = Tkinter.Label(self, textvariable = self.errorstr, font = ('Helvetica', '16', 'bold'))
        display.pack( side = Tkinter.TOP)

        f1 = Tkinter.LabelFrame(self)
# Set Run No.
        f10 = Tkinter.LabelFrame( f1, relief = Tkinter.FLAT)
        rd = Tkinter.Label(f10, text = " Run No.")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.run = Tkinter.Spinbox( f10, from_=1000, to=50000, increment=1, width=5)
        self.run.delete( 0, Tkinter.END)
        self.run.insert( 0, '10371')
        self.run.pack(pady=2,side = Tkinter.LEFT)
        rd = Tkinter.Label( f10, text = "  Temp. Sensor")
        rd.pack(pady=0, side = Tkinter.LEFT)
        # Read Temperature
        ChList = ("A","B","C","C2","D","D2","D3","D4","D5")
        self.Ch = Tkinter.StringVar()
        self.Ch.set(ChList[0])
        self.ChMenu = Tkinter.OptionMenu( f10, self.Ch, *ChList)
        self.ChMenu.pack(pady=0, side = Tkinter.LEFT)

        button2 = Tkinter.Button( f10, text = "Read Temp.", command = self.readtemp)
        button2.pack(pady=0, side = Tkinter.LEFT)
        f10.pack(padx=5, pady=0, fill = Tkinter.BOTH)
        #####################
        #Set Time Parameter
        f11 = Tkinter.LabelFrame( f1, relief = Tkinter.FLAT)
        rd = Tkinter.Label(f11, text = "Time")
        rd.pack(pady=0, side = Tkinter.TOP, anchor = Tkinter.W)
        rd = Tkinter.Label( f11, text = "Start:")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.Sta = Tkinter.Spinbox( f11, from_=0, to=9999999, increment=1, width=7, format='%7.0f')
        self.Sta.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f11, text = "sec.  End:")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.End = Tkinter.Spinbox( f11,from_=0,to=9999999,increment=1,width=7,format='%7.0f')
        self.End.delete( 0, Tkinter.END)
        self.End.insert( 0, '180')
        self.End.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f11, text = "sec.  Time Bin:")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.Bin = Tkinter.Spinbox( f11,from_=1,to=999,increment=1,width=3,format='%3.0f')
        self.Bin.delete( 0, Tkinter.END)
        self.Bin.insert( 0, '60')
        self.Bin.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f11, text = "sec.")
        rd.pack(pady=0, side = Tkinter.LEFT)
        f11.pack(padx=5, pady=2, fill = Tkinter.BOTH)
        ###########
        #Set Tempaerature Parameter
        f12 = Tkinter.LabelFrame( f1, relief = Tkinter.FLAT)
        rd = Tkinter.Label(f12, text = "Temperature")
        rd.pack(pady=0, side = Tkinter.TOP, anchor = Tkinter.W)
        rd = Tkinter.Label( f12, text = "Start:")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.Ti = Tkinter.Spinbox( f12, from_=0, to=9999999, increment=1, width=7, format='%4.0f')
        self.Ti.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f12, text = "K  End:")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.Tf = Tkinter.Spinbox( f12,from_=0,to=9999999,increment=1,width=7,format='%4.0f')
        self.Tf.delete( 0, Tkinter.END)
        self.Tf.insert( 0, '300')
        self.Tf.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f12, text = "K  Bin:")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.TBin = Tkinter.Spinbox( f12,from_=1,to=999,increment=0.05,width=3,format='%2.1f')
        self.TBin.delete( 0, Tkinter.END)
        self.TBin.insert( 0, '1')
        self.TBin.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f12, text = "K")
        rd.pack(pady=0, side = Tkinter.LEFT)
        f12.pack(padx=5, pady=2, fill = Tkinter.BOTH)
        
        self.vReduction = Tkinter.IntVar()
        f13 = Tkinter.LabelFrame( f1, relief = Tkinter.FLAT)
        rd = Tkinter.Label(f13, text = "Reduction from event data ")
        rd.pack(pady=0, side = Tkinter.TOP, anchor = Tkinter.W)
        rd = Tkinter.Radiobutton(f13, text = 'Not Finished', value = 0, variable = self.vReduction)
        #rd = Tkinter.Radiobutton(f13, text = 'Not Finished', value = 0, variable = self.vReduction, command = self.sel_vReduction)
        rd.pack(padx=2, pady=2, side = Tkinter.LEFT)
        rd = Tkinter.Radiobutton(f13, text = 'Finished', value = 1, variable = self.vReduction)
        #rd = Tkinter.Radiobutton(f13, text = 'Finished', value = 1, variable = self.vReduction, command = self.sel_vReduction)
        rd.pack(padx=2, pady=2, side = Tkinter.LEFT)
        f13.pack(padx=5, pady=2, fill = Tkinter.BOTH)
  
        f1.pack(padx=5, pady=5, fill = Tkinter.BOTH)

        f2 = Tkinter.LabelFrame(self)

# Si(111) self.si < 4, Si(333) self.si > 3
# Si111 Select
        self.si = 0
        self.v111 = Tkinter.IntVar()
        self.v333 = Tkinter.IntVar()

        f21 = Tkinter.LabelFrame( f2, relief = Tkinter.FLAT)
        rd = Tkinter.Label(f21, text = "Si(111) : dQ=")
        rd.pack(padx=0,pady=0, side = Tkinter.LEFT)
        self.dqs1 = Tkinter.Spinbox( f21, from_=0.01, to=9.99, increment=0.01, width=4, format='%4.2f')
        self.dqs1.delete( 0, Tkinter.END)
        self.dqs1.insert( 0, '0.05')
        self.dqs1.pack(padx=0,pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label(f21, text = " Qmin=0.0 Qmax=2.0")
        rd.pack(padx=0,pady=0, side = Tkinter.LEFT)
        f21.pack(padx=0,pady=2, fill = Tkinter.BOTH)
        f22 = Tkinter.LabelFrame(f2, relief = Tkinter.FLAT)
        self.v111.set(2)
        rd = Tkinter.Radiobutton(f22, text = 'Fast Chopper OFF (Emin=-200,Emax=50,dE=50μeV) ', value = 0, variable = self.v111, command = self.sel_si111)
        rd.pack(padx=2, pady=2)
        rd = Tkinter.Radiobutton(f22, text = 'Fast Chopper ON  (1cm,1cm,225Hz,Ereso=2.5μeV) ', value = 1, variable = self.v111, command = self.sel_si111)
        rd.pack(padx=2, pady=2)
        rd = Tkinter.Radiobutton(f22, text = 'Fast Chopper ON  (3cm,3cm,225Hz,Ereso=3.5μeV) ', value = 2, variable = self.v111, command = self.sel_si111)
        rd.pack(padx=2, pady=2)
        rd = Tkinter.Radiobutton(f22, text = 'Fast Chopper ON  (1cm,STOP,225Hz,Ereso=3.0μeV)', value = 3, variable = self.v111, command = self.sel_si111)
        rd.pack(padx=2, pady=2)
        rd = Tkinter.Radiobutton(f22, text = 'Fast Chopper ON  (3cm,3cm,300Hz,Ereso=3.1μeV)', value = 4, variable = self.v111, command = self.sel_si111)
        rd.pack(padx=2, pady=2)
        rd.pack(padx=2, pady=2)
        rd = Tkinter.Radiobutton(f22, text = 'Fast Chopper ON  (1cm,1cm,300Hz,Ereso=2.4μeV)', value = 5, variable = self.v111, command = self.sel_si111)
        rd.pack(padx=2, pady=2)
        f22.pack(padx=0,pady=2, fill = Tkinter.BOTH)

        f23 = Tkinter.LabelFrame( f2, relief = Tkinter.FLAT)
        rd = Tkinter.Label( f23, text="  Emin=")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.emin = Tkinter.Spinbox( f23, from_=-999, to=999, increment=1, width=4, format='%4.0f')
        self.emin.delete( 0, Tkinter.END)
        self.emin.insert( 0, '-4')
        self.emin.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f23, text=",Emax=")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.emax = Tkinter.Spinbox( f23, from_=0, to=999, increment=1, width=4, format='%3.0f')
        self.emax.delete( 0, Tkinter.END)
        self.emax.insert( 0, ' 4')
        self.emax.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f23, text=",dE=")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.de = Tkinter.Spinbox( f23, from_=0.01, to=99, increment=0.01, width=5, format='%5.2f')
        self.de.delete( 0, Tkinter.END)
        self.de.insert( 0, ' 2')
        self.de.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f23, text="μeV")
        rd.pack(pady=0, side = Tkinter.LEFT)
        f23.pack(padx=5, pady=2, fill = Tkinter.BOTH)
	
        #[20220403] T0shift Add. by TY 
        f24 = Tkinter.LabelFrame( f2, relief = Tkinter.FLAT)
        rd = Tkinter.Label( f24, text="  T0Shift=")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.T0shift = Tkinter.Spinbox( f24, from_=0, to=400000, increment=1, width=6, format='%4.0f')
        self.T0shift.delete( 0, Tkinter.END)
        self.T0shift.insert( 0, '12325')
        self.T0shift.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f24, text="μs")
        rd.pack(pady=0, side = Tkinter.LEFT)
        f24.pack(padx=5, pady=2, fill = Tkinter.BOTH)


        f2.pack(padx=5, pady=2, fill = Tkinter.BOTH)


#Si 333 GUI
        f3 = Tkinter.LabelFrame(self)
        f31 = Tkinter.LabelFrame(f3, relief = Tkinter.FLAT)
        rd = Tkinter.Label(f31, text = "Si(333) : dQ=")
        rd.pack(padx=0,pady=0, side = Tkinter.LEFT)
        self.dqs3 = Tkinter.Spinbox( f31, from_=0.01, to=9.99, increment=0.01, width=4, format='%4.2f')
        self.dqs3.delete( 0, Tkinter.END)
        self.dqs3.insert( 0, '0.15')
        self.dqs3.pack(padx=0,pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f31, text = " Qmin=0.0 Qmax=6.0")
        rd.pack(padx=0,pady=0, side = Tkinter.LEFT)
        f31.pack(padx=0,pady=2, fill = Tkinter.BOTH)

        f32 = Tkinter.LabelFrame(f3, relief = Tkinter.FLAT)
        self.v333.set(0)
        rd = Tkinter.Radiobutton(f32, text = 'Fast Chopper OFF (Emin=-500,Emax=100,dE=50μeV)', value = 4, variable = self.v333, command = self.sel_si333)
        rd.pack(padx=2, pady=2)
        rd = Tkinter.Radiobutton(f32, text = 'Fast Chopper ON  (1cm,1cm,225Hz)               ', value = 5, variable = self.v333, command = self.sel_si333)
        rd.pack(padx=2, pady=2)
        f32.pack(padx=0,pady=2, fill = Tkinter.BOTH)

        f33 = Tkinter.LabelFrame( f3, relief = Tkinter.FLAT)
        rd = Tkinter.Label( f33, text="  Emin=")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.emin3 = Tkinter.Spinbox( f33, from_=-999, to=999, increment=1, width=4, format='%4.0f')
        self.emin3.delete( 0, Tkinter.END)
        self.emin3.insert( 0, '-500')
        self.emin3.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f33, text=",Emax=")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.emax3 = Tkinter.Spinbox( f33, from_=0, to=999, increment=1, width=4, format='%3.0f')
        self.emax3.delete( 0, Tkinter.END)
        self.emax3.insert( 0, '100')
        self.emax3.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f33, text=",dE=")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.de3 = Tkinter.Spinbox( f33, from_=0.01, to=99, increment=0.01, width=5, format='%5.2f')
        self.de3.delete( 0, Tkinter.END)
        self.de3.insert( 0, ' 50')
        self.de3.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f33, text="μeV")
        rd.pack(pady=0, side = Tkinter.LEFT)
        f33.pack(padx=5, pady=2, fill = Tkinter.BOTH)

        f3.pack(padx=5, pady=5, fill = Tkinter.BOTH)

# Button 
        button1 = Tkinter.Button(self, text = "Reduction Start", command = self.launch)
        button3 = Tkinter.Button(self, text = "Quit", command = self.master.destroy)
        button1.pack(side = Tkinter.LEFT)
        button3.pack(side = Tkinter.RIGHT)
        self.pack()
# self.step == 10: launch stand-by
        self.step = 10

        self.FastChopperUse = True
        self.CommentHeader = "ElasticGUI ->>"

    def sel_si111(self):
        if self.step == 10:
            i = self.v111.get()
        if ( self.si != i):
            self.v333.set(0)
            self.si = i
            self.emin.delete( 0, Tkinter.END)
            self.emax.delete( 0, Tkinter.END)
            self.de.delete( 0, Tkinter.END)
        if ( i == 0):
            self.emin.delete( 0, Tkinter.END)
            self.emax.delete( 0, Tkinter.END)
            self.de.delete( 0, Tkinter.END)
            self.emin.insert( 0, '-200')
            self.emax.insert( 0, ' 50')
            self.de.insert( 0, ' 50')
            self.FastChopperUse = False
            self.T0shift.delete( 0, Tkinter.END)
            self.T0shift.insert( 0, '120')
        elif ( i == 1):
            self.emin.delete( 0, Tkinter.END)
            self.emax.delete( 0, Tkinter.END)
            self.de.delete( 0, Tkinter.END)
            self.emin.insert( 0, '-7')
            self.emax.insert( 0, ' 5')
            self.de.insert( 0, ' 3')
            self.FastChopperUse = True
            self.T0shift.delete( 0, Tkinter.END)
            self.T0shift.insert( 0, '12325')
        elif ( i == 2):
            self.emin.delete( 0, Tkinter.END)
            self.emax.delete( 0, Tkinter.END)
            self.de.delete( 0, Tkinter.END)
            self.emin.insert( 0, '-4')
            self.emax.insert( 0, ' 4')
            self.de.insert( 0, ' 2')
            self.FastChopperUse = True
            self.T0shift.delete( 0, Tkinter.END)
            self.T0shift.insert( 0, '12325')
        elif ( i == 4):
            self.emin.delete( 0, Tkinter.END)
            self.emax.delete( 0, Tkinter.END)
            self.de.delete( 0, Tkinter.END)
            self.emin.insert( 0, '-2')
            self.emax.insert( 0, ' 2')
            self.de.insert( 0, ' 2')
            self.FastChopperUse = True
            self.T0shift.delete( 0, Tkinter.END)
            self.T0shift.insert( 0, '12325')
        elif ( i == 5):
            self.emin.delete( 0, Tkinter.END)
            self.emax.delete( 0, Tkinter.END)
            self.de.delete( 0, Tkinter.END)
            self.emin.insert( 0, '-2')
            self.emax.insert( 0, ' 2')
            self.de.insert( 0, ' 2')
            self.FastChopperUse = True
            self.T0shift.delete( 0, Tkinter.END)
            self.T0shift.insert( 0, '12275')
        else:
            self.emin.delete( 0, Tkinter.END)
            self.emax.delete( 0, Tkinter.END)
            self.de.delete( 0, Tkinter.END)
            self.emin.insert( 0, '-10')
            self.emax.insert( 0, ' 10')
            self.de.insert( 0, ' 5')
            self.FastChopperUse = True
            self.T0shift.delete( 0, Tkinter.END)
            self.T0shift.insert( 0, '12325')

    def sel_si333(self):
        if self.step == 10:
            i = self.v333.get()
            if ( self.si != i):
                self.v111.set(-1)
                self.si = i
                self.emin3.delete( 0, Tkinter.END)
                self.emax3.delete( 0, Tkinter.END)
                self.de3.delete( 0, Tkinter.END)
                self.emax3.insert( 0, '100')
                self.de3.insert( 0, ' 50')
            if ( i == 4):
                self.emin3.insert( 0, '-500')
                self.FastChopperUse = False
            else:
                self.emin3.insert( 0, '-100')
                self.FastChopperUse = True

    def readtemp(self):
        if self.step == 10:
            runNo = int(self.run.get())
            TCh = self.Ch.get()
            print (self.CommentHeader, "RunNo: DNA%6d" %(runNo))
            print (self.CommentHeader, "Temperature Sensor: %s" %(TCh))
            print (self.CommentHeader, "Reading Temperature Start....")
            EC = Cmm.CheckSampleTempByRunNumber( runNo,TCh)
            print (self.CommentHeader, "Reading Temperature end")
            MP.MPlot(EC)
            self.errorstr.set("Temperature run# %d Ch %s" % (runNo, TCh))

    def launch(self):
        self.SetParams()
        self.StartElasticScan()

    def SetParams(self):
        self.RunNo = int(self.run.get())
        self.Tch = self.Ch.get()
        print (self.CommentHeader, "Run No.:", self.RunNo )
        print (self.CommentHeader, "Temperature Sensor: %s" %(self.Tch))
        self.StartTime = self.Sta.get()
        self.EndTime = self.End.get()
        self.TimeBin = self.Bin.get()
        print (self.CommentHeader, "StartTime: %s sec" %(self.StartTime))
        print (self.CommentHeader, "EndTime: %s sec" %(self.EndTime))
        print (self.CommentHeader, "TimeBin: %s sec" %(self.TimeBin))
        print (self.CommentHeader, "FastChopperUse:", self.FastChopperUse)
        self.StartTemp = self.Ti.get()
        self.EndTemp = self.Tf.get()
        self.TempBin = self.TBin.get()
        print (self.CommentHeader, "Start Temperature: %s K" %(self.StartTemp))
        print (self.CommentHeader, "End Temperature: %s K" %(self.EndTemp))
        print (self.CommentHeader, "Temperature BIn: %s K" %(self.TempBin))


        self.Analyzer = 111
        self.Emin = 0
        self.Emax = 0
        self.Ebin = 0
        self.Qmin = 0
        self.Qmax = 0
        self.Qbin = 0
        
        Mode = self.v111.get()
        Mode3 = self.v333.get()
        if Mode3 ==4 or Mode3 ==5:
            self.Analyzer = 333
        print (self.CommentHeader, "Analyzer:%d" %(self.Analyzer))
        
        if Mode <=5 and Mode>=0:
            self.Emin = self.emin.get()
            self.Emax = self.emax.get()
            self.Ebin = self.de.get()
            self.Qmax = 2.0
            self.Qbin = self.dqs1.get()
        elif Mode3 >=4:
            self.Emin = self.emin3.get()
            self.Emax = self.emax3.get() 
            self.Ebin = self.de3.get()
            self.Qmax = 6.0
            self.Qbin = self.dqs3.get()
        else:
            print (self.CommentHeader, "EBin Set Error")
            return -1
        
        self.Emin = float(self.Emin)/1000
        self.Emax = float(self.Emax)/1000
        self.Ebin = float(self.Ebin)/1000
        print (self.CommentHeader, "Emin:%.4f" %(self.Emin))
        print (self.CommentHeader, "Emax:%.4f" %(self.Emax))
        print (self.CommentHeader, "Ebin:%.6f" %(self.Ebin))
        print (self.CommentHeader, "Qmin:%2.1f" %(self.Qmin))
        print (self.CommentHeader, "Qmax:%2.1f" %(self.Qmax))
        print (self.CommentHeader, "Qbin:%s" %(self.Qbin))

    def showplot(self,dat):
        if type(dat) == type(mm.ElementContainer()):
            MP.MPlot(dat)
        elif type(dat) == type(mm.ElementCotainerArray()):
            MP2.M2Plot(dat)
        else:
            print (self.CommentHeader, "dat is not ElementContainer or ElementContainerArray")
            print (self.CommentHeader, "dat type is ", type(dat))

    def StartElasticScan(self):
        print (self.CommentHeader)
        print (self.CommentHeader, "ElasticScan Reduction Start")
        
        inifile = "ESGUIParams.ini"
        iniparams = ESB.SetESiniFile(inifile)
        
        iniparams.setparams("laticetype",self.Analyzer)
        if self.FastChopperUse:
            iniparams.setparams("UseFastChopper","True")
            if self.Analyzer == 111:
#                Mode = self.v111.get()
                iniparams.setparams("t0_offset",self.T0shift.get())
#                print (Mode)
#                if(Mode == 4):
#                    iniparams.setparams("t0_offset","12375")
#                elif(Mode == 5):
#                    iniparams.setparams("t0_offset","12275")
#                    #iniparams.setparams("t0_offset","12365")
#                else:
#                    iniparams.setparams("t0_offset","12325")
            elif self.Analyzer == 333:
                iniparams.setparams("t0_offset","4091")
        else:
            iniparams.setparams("UseFastChopper","False")
            iniparams.setparams("t0_offset",self.T0shift.get())
            #iniparams.setparams("t0_offset","120")
        
        iniparams.writeESinifiles()
        #---LambdaParam
        LambdaCenter = 6.321
        dLambda = 4.15 
        if self.Analyzer == 333:
            LambdaCenter = 3.4 
            dLambda = 4.16 
        
        LambdaParam = "%.4f,%.4f" %(LambdaCenter,dLambda)
        HWParam ="%.4f,%4f,%.6f" %(self.Ebin,self.Emin,self.Emax)
        HWParam2 ="%.4f,%4f" %(self.Emin,self.Emax)
        QParam = "%.2f,%.2f,%s" %(self.Qmin,self.Qmax,self.Qbin)
        print (self.CommentHeader, "HWParam: %s" %( HWParam))
        print (self.CommentHeader, "HWParam2: %s" %(HWParam2))
        print (self.CommentHeader, "LambdaParam: %s"  %(LambdaParam))
        print (self.CommentHeader, "Qparam: %s" %(QParam))
       
        TimeParam = "%s,%s,%s" %(self.StartTime,self.EndTime,self.TimeBin)
        TempParam = "%s,%s,%s" %(self.StartTemp,self.EndTemp,self.TempBin)
        print (self.CommentHeader, "TimeParam: %s" %(TimeParam))
        print (self.CommentHeader, "TempCh: %s" %(self.Tch))
        print (self.CommentHeader, "TempParam: %s" %(TempParam))

        #dataPath = "/home/dnaadmin/DNA/tmp/" #T.Yamada[130218]
        dataPath = os.path.join(mu.UtsusemiEnvGetUserDir(), "DNA", "tmp")  #[inamura 230301]
        ECMbstName = "ECM_%d_ES.bst" %(self.RunNo)
        
        FileHeader = "ESGUIRun"
        ESName = "%s%dES.txt" % (FileHeader, self.RunNo)
        ESTimeName = "%s%dEStime.txt" % (FileHeader, self.RunNo)
        TIName = "%s%dTI.txt" % (FileHeader, self.RunNo)
        TITimeName = "%s%dTItime.txt" % (FileHeader, self.RunNo)
        TempLogName = "%s%dTempLog.txt" %(FileHeader, self.RunNo)
    
        #copyresult, messg =CopyEventData(int(self.RunNo))
        #print (messg) 

        #print (type(self.RunNo), self.RunNo)
        T0M.MakeT0file(str("%06d" %int(self.RunNo)))

        import ElasticScan 
        #Make Case Info xml
        ESP3.MakeCaseInfoXML(float(self.StartTime), float(self.EndTime), float(self.TimeBin))
        print (self.CommentHeader, "ECM make start")
        ECM = ESP3.ESTimeSlice(RunNo=int(self.RunNo), dataFolder="/data", HWParam=HWParam, LambdaParam=LambdaParam, QParam=QParam, inifile=inifile)
        #ECM = ElasticScan.GetQETimeECMMulth(self.RunNo, HWParam, LambdaParam, QParam, TimeParam, inifile)
        print (self.CommentHeader, "ECM make end")
        
        #ECM = ElasticScan.ReadECM(ECMbstName)
        EC = ElasticScan.ElasticScanTotalIntensity(ECM, HWParam2, QParam, inifile)
        print (self.CommentHeader, "Total Intensity vs Time")
        ElasticScan.OutputECText(EC, TITimeName)
        ElasticScan.OutPutElasticScanECM(ECM)
        del ECM
        del ElasticScan
        self.showplot(EC)
        print ("Finished")
        return -1
        #----------------- 
        #u2 vs time is calculated
        ECA = ElasticScan.ElasticScanQ2LnI(ECM, HWParam2, QParam, inifile)
        EC1 = ElasticScan.ElasticScanFit(ECA, "p1")
        ElasticScan.OutputECText(EC1, ESTimeName)
        self.showplot(EC1)
        del ECM
        del ECA
        #Time is converted to temperature.
        EC2 = ElasticScan.ElasticScanCheckTempNew(EC, self.Tch)
        EC4 = ElasticScan.ElasticScanXAxisChangeToTemp(EC, EC2, TempParam)
        ElasticScan.OutputECText(EC2,TempLogName) #Temperature log
        ElasticScan.OutputECText(EC4, TIName) #TotalIntensity vs Temperature
        EC3 = ElasticScan.ElasticScanXAxisChangeToTemp(EC1, EC2, TempParam)
        ElasticScan.OutputECText(EC3, ESName) #u2 vs Temperature
        print (self.CommentHeader, "ElasticScan Reduction End")
        print (self.CommentHeader, "I(Q) vs time is saved as \n%sRun%sIQ.txt\n" %(dataPath,self.RunNo) )
        print (self.CommentHeader, "u2 is saved as \n%s%s\n%s%s\n" %(dataPath, ESName, dataPath, ESTimeName))
        print (self.CommentHeader, "Total intensity is saved as \n%s%s\n%s%s\n" %(dataPath, TIName, dataPath, TITimeName))
        print (self.CommentHeader, "Temperature Log is saved as \n%s%s\n" %(dataPath, TempLogName))
        
        self.showplot(EC3)

def CopyEventData(RunNo):
    command = "ssh dnaadmin@192.168.1.10 /home/dnaadmin/Yamada/01_CopyEventData/CopyEventEdata.sh %d" %(RunNo)
    args = command.split(" ")
    proc = subprocess.Popen(args, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
    stdout_data, stderr_data = proc.communicate()
    print ("stdout_data >>", stdout_data)
    print ("stderr_data >>", stderr_data.replace("\n",""))
    result = proc.returncode
    #proc.close()
    if result == 1:
        messg =  "DNA%06d_* is not found on /data and /ifs_data/DNA/DNA" %RunNo
    else:
        messg =  "DNA%06d_* is ready for the reduction." %RunNo
    return result, messg


if __name__ == "__main__":
    FastChopperUse = False
    app = App()
    app.mainloop()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import multiprocessing
from optparse import OptionParser
import os
import re
import platform
import sys
PYSIDEVER = 6
try:
    from PySide6 import QtCore, QtGui, QtWidgets
    from PySide6.QtWidgets import QApplication, QFileDialog, QListWidgetItem, QMessageBox
    from PySide6.QtCore import QFile
    from ui6_DlgSetUtsusemiEnv import Ui_MainWindow
except:
    from PySide2 import QtCore, QtGui, QtWidgets
    # from PySide2.QtUiTools import QUiLoader
    from PySide2.QtWidgets import QApplication, QFileDialog, QListWidgetItem, QMessageBox
    from PySide2.QtCore import QFile
    from ui2_DlgSetUtsusemiEnv import Ui_MainWindow
    PYSIDEVER = 2

if sys.platform == 'win32':
    HOME = os.environ.get("USERPROFILE")
elif sys.platform == 'darwin':
    HOME = os.environ.get("HOME")
elif sys.platform == 'linux2':
    HOME = os.environ.get("HOME")
else:
    HOME = os.curdir

# UTSUSEMI_DATA_DIRとUTSUSEMI_USR_DIRのベースディレクトリの初期値
BASE_DIR = os.path.join(HOME, "Documents", "Utsusemi")

# "utsusemienv.cfg"のパス
FILE_CONFIG = os.environ.get("CONFFILE_ENV")
FILE_CONFIG_DIR = os.path.dirname(FILE_CONFIG)

# カレントディレクトリ
CURRENT_DIR = os.getcwd()

# 設定対象の環境変数
UTSUSEMI_INST_CODE = "UTSUSEMI_INST_CODE"
UTSUSEMI_LOG_QUIET = "UTSUSEMI_LOG_QUIET"
UTSUSEMI_DEBUGMODE = "UTSUSEMI_DEBUGMODE"
UTSUSEMI_MULTH = "UTSUSEMI_MULTH"
UTSUSEMI_DATA_DIR = "UTSUSEMI_DATA_DIR"
UTSUSEMI_USR_DIR = "UTSUSEMI_USR_DIR"
UTSUSEMI_WORK_DIR = "UTSUSEMI_WORK_DIR"
UTSUSEMI_DEFAULT_FONTSIZE = "UTSUSEMI_DEFAULT_FONTSIZE"

# 環境変数のリスト
VARS = [
    UTSUSEMI_INST_CODE,
    UTSUSEMI_LOG_QUIET,
    UTSUSEMI_DEBUGMODE,
    UTSUSEMI_MULTH,
    UTSUSEMI_DATA_DIR,
    UTSUSEMI_USR_DIR,
    UTSUSEMI_WORK_DIR,
    UTSUSEMI_DEFAULT_FONTSIZE
]

# ビームライン選択用の辞書
BEAMLINES = {
    "SIK": {'line': 1, 'name': "4SEASONS"},
    "DNA": {'line': 2, 'name': "DNA"},
    "NBR": {'line': 10, 'name': "Noboru"},
    "HPN": {'line': 11, 'name': "PLANET"},
    "AMR": {'line': 14, 'name': "AMATERAS"},
    "SAS": {'line': 15, 'name': "TAIKAN"},
    "ENG": {'line': 19, 'name': "TAKUMI"},
}

"""
if sys.platform == 'darwin' or sys.platform == 'linux':
    # MacとLinuxではDNAも使用可能(Noboru is only command line)
    BEAMLINES["DNA"] = {'line': 2, 'name': "DNA"}
    # BEAMLINES["NBR"] = {'line': 10, 'name': "Noboru"}
"""

# Y/N←→真偽値 変換用の辞書
LOG_QUIET = {"Y": True, "N": False}


class Param:
    def __init__(self):
        self.data_dir = os.path.join(BASE_DIR, "DATA")
        self.usr_dir = os.path.join(BASE_DIR, "HOME")
        self.work_dir = self.usr_dir
        self.multh = 2 if multiprocessing.cpu_count() >= 2 else 1
        self.log_quiet = True
        self.debug_mode = False
        self.inst_code = "SIK"
        self.def_fontsize = 12
        self.loaded = {x: False for x in VARS}

    def load(self, path):
        if not os.path.isfile(path):
            return
        try:
            with open(path) as f:
                d = {}
                for x in f.readlines():
                    if "=" in x:
                        key, val = x.strip().split("=", 1)
                        d[key] = re.sub(r'[\"\'\n\r]+', '', val)
                for key, val in d.items():
                    msg = ""
                    if key == UTSUSEMI_MULTH:
                        try:
                            self.multh = int(val)
                        except:
                            msg = "MULTH: not an integer: >>>" + val + "<<<"
                    elif key == UTSUSEMI_INST_CODE:
                        self.inst_code = val
                    elif key == UTSUSEMI_LOG_QUIET:
                        if val in ("Y", "N"):
                            self.log_quiet = ("Y" == val)
                        else:
                            msg = "LOG_QUIET: illegal value: >>>" + val + "<<<"
                    elif key == UTSUSEMI_DEBUGMODE:
                        if val in ("Y", "N"):
                            self.debug_mode = ("Y" == val)
                        else:
                            msg = "DEBUGMODE: illegal value: >>>" + val + "<<<"
                    elif key == UTSUSEMI_USR_DIR:
                        self.usr_dir = val
                    elif key == UTSUSEMI_DATA_DIR:
                        self.data_dir = val.strip('"')
                    elif key == UTSUSEMI_WORK_DIR:
                        self.work_dir = val.strip('"')
                    elif key == UTSUSEMI_DEFAULT_FONTSIZE:
                        self.def_fontsize = int(val)

                    if key in self.loaded.keys() and not msg:
                        self.loaded[key] = True
                    if msg:
                        QMessageBox.warning(None, "PARAMETER ERROR", msg)
        except Exception as e:
            QMessageBox.warning(None, "LOAD ERROR",
                                "unknown load error: " + str(e))

    def bad_paths(self):
        d = self.data_dir
        u = self.usr_dir
        w = self.work_dir
        ret = []
        if not os.path.exists(d) or not os.path.isdir(d):
            ret.append(d)
        if not os.path.exists(u) or not os.path.isdir(u):
            ret.append(u)
        if not os.path.exists(w) or not os.path.isdir(w):
            ret.append(w)
        return ret

    def dump(self, path):
        bad = self.bad_paths()
        if bad:
            return bad
        with open(path, "w") as f:
            f.write("%s=%s\n" % (UTSUSEMI_INST_CODE, self.inst_code))
            f.write("%s=%s\n" % (UTSUSEMI_LOG_QUIET, ("Y" if self.log_quiet else "N")))
            f.write("%s=%s\n" % (UTSUSEMI_DEBUGMODE, ("Y" if self.debug_mode else "N")))
            f.write("%s=%d\n" % (UTSUSEMI_MULTH, self.multh))
            f.write("%s=\"%s\"\n" % (UTSUSEMI_DATA_DIR, self.data_dir))
            f.write("%s=\"%s\"\n" % (UTSUSEMI_USR_DIR, self.usr_dir))
            f.write("%s=\"%s\"\n" % (UTSUSEMI_WORK_DIR, self.work_dir))
            f.write("%s=%d\n" % (UTSUSEMI_DEFAULT_FONTSIZE, self.def_fontsize))
        return []


class Main(QtWidgets.QMainWindow):
    def __init__(self, parent):
        # file = QFile(path)
        # file.open(QFile.ReadOnly)
        # loader = QUiLoader()
        # self = loader.load(file)
        super(Main, self).__init__(parent)
        self.ui = Ui_MainWindow()
        if self.ui is None:
            return
        self.ui.setupUi(self)
        self.param = Param()
        self.getGui()
        self.setup()
        self.connections()
        self.show()

    def getGui(self):
        self.beamList = self.findChild(QtWidgets.QListWidget, 'beamList')
        self.cbLogQuiet = self.findChild(QtWidgets.QCheckBox, 'cbLogQuiet')
        self.spnFontSize = self.findChild(QtWidgets.QSpinBox, 'spnFontSize')
        self.spnNumThread = self.findChild(QtWidgets.QSpinBox, 'spnNumThread')
        self.cbDebugMode = self.findChild(QtWidgets.QCheckBox, 'cbDebugMode')
        self.buttonBox = self.findChild(QtWidgets.QDialogButtonBox, 'buttonBox')
        self.txtDataDir = self.findChild(QtWidgets.QLineEdit, 'txtDataDir')
        self.btDataDir = self.findChild(QtWidgets.QPushButton, 'btDataDir')
        self.txtUsrDir = self.findChild(QtWidgets.QLineEdit, 'txtUsrDir')
        self.btUsrDir = self.findChild(QtWidgets.QPushButton, 'btUsrDir')
        self.txtWorkDir = self.findChild(QtWidgets.QLineEdit, 'txtWorkDir')
        self.btWorkDir = self.findChild(QtWidgets.QPushButton, 'btWorkDir')
        
    def setup(self):
        self.items = {}
        for x, vs in BEAMLINES.items():
            s = "BL" + str(vs['line']).zfill(2) + ":" + \
                x + "(" + vs['name'] + ")"
            item = QListWidgetItem(s)
            self.beamList.addItem(item)
            self.items[x] = item
        # self.beamList.setCurrentItem(self.items["SIK"])

        self.spnNumThread.setRange(1, multiprocessing.cpu_count())
        if FILE_CONFIG is None:
            msg = """You need to define an environment variable 'CONFFILE_ENV'
whose value is the filepath of utsusemienv.cfg"""
            QMessageBox.critical(self, "Missing CONFFILE_ENV", msg)
            sys.exit(1)
        self.param.load(FILE_CONFIG)
        self.txtDataDir.setText(self.param.data_dir)
        self.txtUsrDir.setText(self.param.usr_dir)
        self.txtWorkDir.setText(self.param.work_dir)
        self.spnNumThread.setValue(self.param.multh)
        self.spnFontSize.setValue(self.param.def_fontsize)
        if self.param.log_quiet:
            print("QUIET")
            self.cbLogQuiet.setChecked(True)
        if self.param.debug_mode:
            print("DEBUG_MODE")
            self.cbDebugMode.setChecked(True)
        self.beamList.setCurrentItem(self.items[self.param.inst_code])
        # self.beamList.setItemSelected(self.items[self.param.inst_code],True)
        self.items[self.param.inst_code].setSelected(True)

    def connections(self):
        self.buttonBox.accepted.connect(self.save)
        self.buttonBox.rejected.connect(self.quit)
        self.btDataDir.clicked.connect(self.fdData)
        self.btUsrDir.clicked.connect(self.fdUsr)
        self.btWorkDir.clicked.connect(self.fdWork)

    def quit(self, force=True):
        if force:
            app.exit()
            self.close()

    def fdData(self):
        ret = QFileDialog.getExistingDirectory(
            self, "DD", self.param.data_dir)
        if len(ret) > 0:
            UTSUSEMI_DATA_DIR = os.path.normpath(ret)
            self.txtDataDir.setText(UTSUSEMI_DATA_DIR)

    def fdUsr(self):
        ret = QFileDialog.getExistingDirectory(
            self, "DD", self.param.usr_dir)
        if len(ret) > 0:
            UTSUSEMI_USR_DIR = os.path.normpath(ret)
            self.txtUsrDir.setText(UTSUSEMI_USR_DIR)

    def fdWork(self):
        ret = QFileDialog.getExistingDirectory(
            self, "DD", self.param.work_dir)
        if len(ret) > 0:
            UTSUSEMI_WORK_DIR = os.path.normpath(ret)
            self.txtWorkDir.setText(UTSUSEMI_WORK_DIR)

    def save(self):
        d1 = self.txtDataDir.text()
        d2 = self.txtUsrDir.text()
        d3 = self.txtWorkDir.text()
        for d in [d1, d2, d3]:
            if not os.path.exists(d):
                msg = "Not found given folder ({}) . Do you want to make it ?".format(
                    d)
                ret = QMessageBox().warning(None, u"Warning", msg,
                                            QMessageBox.Ok, QMessageBox.Cancel)
                if ret == QMessageBox.Ok:
                    try:
                        os.makedirs(d)
                    except:
                        QMessageBox.warning(None, "ERROR", "No permitted.")
                        return
                else:
                    self.quit()
        self.param.data_dir = d1
        self.param.usr_dir = d2
        self.param.work_dir = d3
        if not os.path.exists(os.path.join(d2, "ana", "xml")):
            try:
                os.makedirs(os.path.join(d2, "ana", "xml"))
            except:
                QMessageBox.warning(
                    None, "WARNING", "Failed to make ana/xml ", QMessageBox.Ok)
        if not os.path.exists(os.path.join(d2, "ana", "tmp")):
            try:
                os.makedirs(os.path.join(d2, "ana", "tmp"))
            except:
                QMessageBox.warning(
                    None, "WARNING", "Failed to make ana/tmp ", QMessageBox.Ok)
        self.param.log_quiet = self.cbLogQuiet.isChecked()
        self.param.debug_mode = self.cbDebugMode.isChecked()
        self.param.multh = self.spnNumThread.value()
        self.param.def_fontsize = int(self.spnFontSize.value())
        citem = self.beamList.currentItem()
        for k, item in self.items.items():
            if item.text == citem.text:
                self.param.inst_code = k
        if not os.path.exists(FILE_CONFIG_DIR):
            os.makedirs(FILE_CONFIG_DIR)
        bad = self.param.dump(FILE_CONFIG)
        if not bad:
            msg = "new settings were saved"
            QMessageBox.information(self, "Confirmation", msg)
            self.quit()
        msg = "illegal path(s): " + ",".join(bad)
        QMessageBox.warning(self, "ILLEGAL PATHS", msg)


if __name__ == '__main__':
    parser = OptionParser(usage=u'%prog [OPTIONS]')
    parser.add_option(
        '-a', '--alert',
        help='Display alert.',
        action='store_true',
        default=False,
        dest='isAlert'
    )
    parser.add_option(
        '-m', '--message',
        help='Alert message to be shown',
        default=None,
        dest='message')
    (option, args) = parser.parse_args()

    if PYSIDEVER != 6:
        if hasattr(QtCore.Qt, 'AA_EnableHighDpiScaling'):
            QtWidgets.QApplication.setAttribute(
                QtCore.Qt.AA_EnableHighDpiScaling, True)

    app = QApplication(sys.argv)

    if option.isAlert:
        msg = "\n".join([option.message,
                         "",
                         "Run the [Utsusemi Environment Settings] and set up the environment variables."])
        QMessageBox.critical(None, "Missing CONFFILE_ENV", msg)
        sys.exit(1)

    if FILE_CONFIG is None:
        msg = "\n".join(["You need to define an environment variable 'CONFFILE_ENV'"
                         "whose value is the filepath of utsusemienv.cfg"])
        QMessageBox.critical(None, "Missing CONFFILE_ENV", msg)
        sys.exit(1)

    win = Main(None)
    if PYSIDEVER == 6:
        sys.exit(app.exec())
    else:
        sys.exit(app.exec_())



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'SAS_UBSlicer.ui'
##
## Created by: Qt User Interface Compiler version 6.4.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QFrame, QGroupBox,
    QLabel, QLineEdit, QMainWindow, QMenu,
    QMenuBar, QPushButton, QSizePolicy, QStatusBar,
    QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(594, 469)
        self.actionLoad = QAction(MainWindow)
        self.actionLoad.setObjectName(u"actionLoad")
        self.actionExit = QAction(MainWindow)
        self.actionExit.setObjectName(u"actionExit")
        self.actionLoad_Param = QAction(MainWindow)
        self.actionLoad_Param.setObjectName(u"actionLoad_Param")
        self.actionSave_Param = QAction(MainWindow)
        self.actionSave_Param.setObjectName(u"actionSave_Param")
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.PanelProj = QGroupBox(self.centralwidget)
        self.PanelProj.setObjectName(u"PanelProj")
        self.PanelProj.setGeometry(QRect(10, 130, 541, 111))
        self.txt_proj_ax11 = QLineEdit(self.PanelProj)
        self.txt_proj_ax11.setObjectName(u"txt_proj_ax11")
        self.txt_proj_ax11.setGeometry(QRect(40, 40, 55, 20))
        self.label_pr5 = QLabel(self.PanelProj)
        self.label_pr5.setObjectName(u"label_pr5")
        self.label_pr5.setGeometry(QRect(10, 40, 31, 17))
        self.txt_proj_ax12 = QLineEdit(self.PanelProj)
        self.txt_proj_ax12.setObjectName(u"txt_proj_ax12")
        self.txt_proj_ax12.setGeometry(QRect(100, 40, 55, 20))
        self.txt_proj_ax13 = QLineEdit(self.PanelProj)
        self.txt_proj_ax13.setObjectName(u"txt_proj_ax13")
        self.txt_proj_ax13.setGeometry(QRect(160, 40, 55, 20))
        self.label_pr6 = QLabel(self.PanelProj)
        self.label_pr6.setObjectName(u"label_pr6")
        self.label_pr6.setGeometry(QRect(10, 60, 31, 17))
        self.txt_proj_ax23 = QLineEdit(self.PanelProj)
        self.txt_proj_ax23.setObjectName(u"txt_proj_ax23")
        self.txt_proj_ax23.setGeometry(QRect(160, 60, 55, 20))
        self.txt_proj_ax21 = QLineEdit(self.PanelProj)
        self.txt_proj_ax21.setObjectName(u"txt_proj_ax21")
        self.txt_proj_ax21.setGeometry(QRect(40, 60, 55, 20))
        self.txt_proj_ax22 = QLineEdit(self.PanelProj)
        self.txt_proj_ax22.setObjectName(u"txt_proj_ax22")
        self.txt_proj_ax22.setGeometry(QRect(100, 60, 55, 20))
        self.label_pr7 = QLabel(self.PanelProj)
        self.label_pr7.setObjectName(u"label_pr7")
        self.label_pr7.setGeometry(QRect(10, 80, 31, 17))
        self.txt_proj_t1 = QLineEdit(self.PanelProj)
        self.txt_proj_t1.setObjectName(u"txt_proj_t1")
        self.txt_proj_t1.setGeometry(QRect(240, 40, 91, 20))
        self.txt_proj_t2 = QLineEdit(self.PanelProj)
        self.txt_proj_t2.setObjectName(u"txt_proj_t2")
        self.txt_proj_t2.setGeometry(QRect(240, 60, 91, 20))
        self.label_pr1 = QLabel(self.PanelProj)
        self.label_pr1.setObjectName(u"label_pr1")
        self.label_pr1.setGeometry(QRect(40, 20, 55, 20))
        self.label_pr1.setAlignment(Qt.AlignCenter)
        self.label_pr2 = QLabel(self.PanelProj)
        self.label_pr2.setObjectName(u"label_pr2")
        self.label_pr2.setGeometry(QRect(100, 20, 55, 20))
        self.label_pr2.setAlignment(Qt.AlignCenter)
        self.label_pr3 = QLabel(self.PanelProj)
        self.label_pr3.setObjectName(u"label_pr3")
        self.label_pr3.setGeometry(QRect(160, 20, 55, 20))
        self.label_pr3.setAlignment(Qt.AlignCenter)
        self.label_pr4 = QLabel(self.PanelProj)
        self.label_pr4.setObjectName(u"label_pr4")
        self.label_pr4.setGeometry(QRect(244, 20, 91, 20))
        self.label_pr4.setAlignment(Qt.AlignCenter)
        self.txt_proj_t3 = QLineEdit(self.PanelProj)
        self.txt_proj_t3.setObjectName(u"txt_proj_t3")
        self.txt_proj_t3.setGeometry(QRect(240, 80, 91, 20))
        self.txt_proj_ax33 = QLineEdit(self.PanelProj)
        self.txt_proj_ax33.setObjectName(u"txt_proj_ax33")
        self.txt_proj_ax33.setGeometry(QRect(160, 80, 55, 20))
        self.txt_proj_ax31 = QLineEdit(self.PanelProj)
        self.txt_proj_ax31.setObjectName(u"txt_proj_ax31")
        self.txt_proj_ax31.setGeometry(QRect(40, 80, 55, 20))
        self.txt_proj_ax32 = QLineEdit(self.PanelProj)
        self.txt_proj_ax32.setObjectName(u"txt_proj_ax32")
        self.txt_proj_ax32.setGeometry(QRect(100, 80, 55, 20))
        self.btProj = QPushButton(self.PanelProj)
        self.btProj.setObjectName(u"btProj")
        self.btProj.setGeometry(QRect(420, 80, 99, 27))
        self.btClr_proj = QPushButton(self.PanelProj)
        self.btClr_proj.setObjectName(u"btClr_proj")
        self.btClr_proj.setGeometry(QRect(420, 40, 51, 27))
        self.txt_proj_u1 = QLineEdit(self.PanelProj)
        self.txt_proj_u1.setObjectName(u"txt_proj_u1")
        self.txt_proj_u1.setGeometry(QRect(340, 40, 61, 20))
        self.txt_proj_u2 = QLineEdit(self.PanelProj)
        self.txt_proj_u2.setObjectName(u"txt_proj_u2")
        self.txt_proj_u2.setGeometry(QRect(340, 60, 61, 20))
        self.txt_proj_u3 = QLineEdit(self.PanelProj)
        self.txt_proj_u3.setObjectName(u"txt_proj_u3")
        self.txt_proj_u3.setGeometry(QRect(340, 80, 61, 20))
        self.label_pr4_2 = QLabel(self.PanelProj)
        self.label_pr4_2.setObjectName(u"label_pr4_2")
        self.label_pr4_2.setGeometry(QRect(340, 20, 51, 20))
        self.label_pr4_2.setAlignment(Qt.AlignCenter)
        self.PanelSlice = QGroupBox(self.centralwidget)
        self.PanelSlice.setObjectName(u"PanelSlice")
        self.PanelSlice.setGeometry(QRect(10, 250, 571, 161))
        self.PanelSlice.setFlat(False)
        self.label_sl4 = QLabel(self.PanelSlice)
        self.label_sl4.setObjectName(u"label_sl4")
        self.label_sl4.setGeometry(QRect(0, 70, 31, 17))
        self.txt_sl_ax11 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax11.setObjectName(u"txt_sl_ax11")
        self.txt_sl_ax11.setGeometry(QRect(30, 70, 60, 20))
        self.label_sl5 = QLabel(self.PanelSlice)
        self.label_sl5.setObjectName(u"label_sl5")
        self.label_sl5.setGeometry(QRect(100, 70, 16, 17))
        self.txt_sl_ax12 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax12.setObjectName(u"txt_sl_ax12")
        self.txt_sl_ax12.setGeometry(QRect(120, 70, 60, 20))
        self.btAutoFillAx1 = QPushButton(self.PanelSlice)
        self.btAutoFillAx1.setObjectName(u"btAutoFillAx1")
        self.btAutoFillAx1.setGeometry(QRect(190, 70, 24, 24))
        self.cb_sl_ax1 = QComboBox(self.PanelSlice)
        self.cb_sl_ax1.addItem("")
        self.cb_sl_ax1.addItem("")
        self.cb_sl_ax1.addItem("")
        self.cb_sl_ax1.setObjectName(u"cb_sl_ax1")
        self.cb_sl_ax1.setGeometry(QRect(230, 70, 100, 25))
        self.txt_sl_ax13 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax13.setObjectName(u"txt_sl_ax13")
        self.txt_sl_ax13.setGeometry(QRect(340, 70, 60, 20))
        self.btAutoFillAx2 = QPushButton(self.PanelSlice)
        self.btAutoFillAx2.setObjectName(u"btAutoFillAx2")
        self.btAutoFillAx2.setGeometry(QRect(190, 100, 24, 24))
        self.label_sl7 = QLabel(self.PanelSlice)
        self.label_sl7.setObjectName(u"label_sl7")
        self.label_sl7.setGeometry(QRect(100, 100, 16, 17))
        self.txt_sl_ax22 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax22.setObjectName(u"txt_sl_ax22")
        self.txt_sl_ax22.setGeometry(QRect(120, 100, 60, 20))
        self.txt_sl_ax23 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax23.setObjectName(u"txt_sl_ax23")
        self.txt_sl_ax23.setGeometry(QRect(340, 100, 60, 20))
        self.cb_sl_ax2 = QComboBox(self.PanelSlice)
        self.cb_sl_ax2.addItem("")
        self.cb_sl_ax2.addItem("")
        self.cb_sl_ax2.addItem("")
        self.cb_sl_ax2.setObjectName(u"cb_sl_ax2")
        self.cb_sl_ax2.setGeometry(QRect(230, 100, 100, 25))
        self.label_sl6 = QLabel(self.PanelSlice)
        self.label_sl6.setObjectName(u"label_sl6")
        self.label_sl6.setGeometry(QRect(0, 100, 31, 17))
        self.txt_sl_ax21 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax21.setObjectName(u"txt_sl_ax21")
        self.txt_sl_ax21.setGeometry(QRect(30, 100, 60, 20))
        self.btAutoFillAx3 = QPushButton(self.PanelSlice)
        self.btAutoFillAx3.setObjectName(u"btAutoFillAx3")
        self.btAutoFillAx3.setGeometry(QRect(190, 130, 24, 24))
        self.label_sl9 = QLabel(self.PanelSlice)
        self.label_sl9.setObjectName(u"label_sl9")
        self.label_sl9.setGeometry(QRect(100, 130, 16, 17))
        self.txt_sl_ax32 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax32.setObjectName(u"txt_sl_ax32")
        self.txt_sl_ax32.setGeometry(QRect(120, 130, 60, 20))
        self.txt_sl_ax33 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax33.setObjectName(u"txt_sl_ax33")
        self.txt_sl_ax33.setGeometry(QRect(340, 130, 60, 20))
        self.cb_sl_ax3 = QComboBox(self.PanelSlice)
        self.cb_sl_ax3.addItem("")
        self.cb_sl_ax3.addItem("")
        self.cb_sl_ax3.addItem("")
        self.cb_sl_ax3.setObjectName(u"cb_sl_ax3")
        self.cb_sl_ax3.setGeometry(QRect(230, 130, 100, 25))
        self.label_sl8 = QLabel(self.PanelSlice)
        self.label_sl8.setObjectName(u"label_sl8")
        self.label_sl8.setGeometry(QRect(0, 130, 31, 17))
        self.txt_sl_ax31 = QLineEdit(self.PanelSlice)
        self.txt_sl_ax31.setObjectName(u"txt_sl_ax31")
        self.txt_sl_ax31.setGeometry(QRect(30, 130, 60, 20))
        self.btAutoFill = QPushButton(self.PanelSlice)
        self.btAutoFill.setObjectName(u"btAutoFill")
        self.btAutoFill.setGeometry(QRect(130, 40, 51, 27))
        self.label_sl1 = QLabel(self.PanelSlice)
        self.label_sl1.setObjectName(u"label_sl1")
        self.label_sl1.setGeometry(QRect(60, 40, 51, 17))
        self.label_sl2 = QLabel(self.PanelSlice)
        self.label_sl2.setObjectName(u"label_sl2")
        self.label_sl2.setGeometry(QRect(240, 40, 81, 17))
        self.label_sl3 = QLabel(self.PanelSlice)
        self.label_sl3.setObjectName(u"label_sl3")
        self.label_sl3.setGeometry(QRect(350, 40, 51, 17))
        self.btSlice = QPushButton(self.PanelSlice)
        self.btSlice.setObjectName(u"btSlice")
        self.btSlice.setGeometry(QRect(420, 120, 120, 30))
        self.PanelUB = QGroupBox(self.centralwidget)
        self.PanelUB.setObjectName(u"PanelUB")
        self.PanelUB.setGeometry(QRect(10, 20, 541, 101))
        self.txt12_ub = QLineEdit(self.PanelUB)
        self.txt12_ub.setObjectName(u"txt12_ub")
        self.txt12_ub.setGeometry(QRect(170, 30, 60, 20))
        self.txt22_ub = QLineEdit(self.PanelUB)
        self.txt22_ub.setObjectName(u"txt22_ub")
        self.txt22_ub.setGeometry(QRect(170, 50, 60, 20))
        self.txt23_ub = QLineEdit(self.PanelUB)
        self.txt23_ub.setObjectName(u"txt23_ub")
        self.txt23_ub.setGeometry(QRect(240, 50, 60, 20))
        self.txt21_ub = QLineEdit(self.PanelUB)
        self.txt21_ub.setObjectName(u"txt21_ub")
        self.txt21_ub.setGeometry(QRect(100, 50, 60, 20))
        self.txt32_ub = QLineEdit(self.PanelUB)
        self.txt32_ub.setObjectName(u"txt32_ub")
        self.txt32_ub.setGeometry(QRect(170, 70, 60, 20))
        self.txt11_ub = QLineEdit(self.PanelUB)
        self.txt11_ub.setObjectName(u"txt11_ub")
        self.txt11_ub.setGeometry(QRect(100, 30, 60, 20))
        self.txt33_ub = QLineEdit(self.PanelUB)
        self.txt33_ub.setObjectName(u"txt33_ub")
        self.txt33_ub.setGeometry(QRect(240, 70, 60, 20))
        self.txt13_ub = QLineEdit(self.PanelUB)
        self.txt13_ub.setObjectName(u"txt13_ub")
        self.txt13_ub.setGeometry(QRect(240, 30, 60, 20))
        self.txt31_ub = QLineEdit(self.PanelUB)
        self.txt31_ub.setObjectName(u"txt31_ub")
        self.txt31_ub.setGeometry(QRect(100, 70, 60, 20))
        self.label_ub1 = QLabel(self.PanelUB)
        self.label_ub1.setObjectName(u"label_ub1")
        self.label_ub1.setGeometry(QRect(20, 30, 41, 21))
        self.label_ub2 = QLabel(self.PanelUB)
        self.label_ub2.setObjectName(u"label_ub2")
        self.label_ub2.setGeometry(QRect(20, 50, 41, 21))
        self.label_ub3 = QLabel(self.PanelUB)
        self.label_ub3.setObjectName(u"label_ub3")
        self.label_ub3.setGeometry(QRect(20, 70, 41, 21))
        self.label_ub6 = QLabel(self.PanelUB)
        self.label_ub6.setObjectName(u"label_ub6")
        self.label_ub6.setGeometry(QRect(360, 50, 41, 21))
        self.label_ub7 = QLabel(self.PanelUB)
        self.label_ub7.setObjectName(u"label_ub7")
        self.label_ub7.setGeometry(QRect(360, 70, 41, 21))
        self.label_ub5 = QLabel(self.PanelUB)
        self.label_ub5.setObjectName(u"label_ub5")
        self.label_ub5.setGeometry(QRect(360, 30, 41, 21))
        self.label_ub4 = QLabel(self.PanelUB)
        self.label_ub4.setObjectName(u"label_ub4")
        self.label_ub4.setGeometry(QRect(60, 50, 21, 17))
        self.line_ub1 = QFrame(self.PanelUB)
        self.line_ub1.setObjectName(u"line_ub1")
        self.line_ub1.setGeometry(QRect(10, 30, 3, 61))
        self.line_ub1.setFrameShape(QFrame.VLine)
        self.line_ub1.setFrameShadow(QFrame.Sunken)
        self.line_ub2 = QFrame(self.PanelUB)
        self.line_ub2.setObjectName(u"line_ub2")
        self.line_ub2.setGeometry(QRect(50, 30, 3, 61))
        self.line_ub2.setFrameShape(QFrame.VLine)
        self.line_ub2.setFrameShadow(QFrame.Sunken)
        self.line_ub3 = QFrame(self.PanelUB)
        self.line_ub3.setObjectName(u"line_ub3")
        self.line_ub3.setGeometry(QRect(90, 30, 3, 61))
        self.line_ub3.setFrameShape(QFrame.VLine)
        self.line_ub3.setFrameShadow(QFrame.Sunken)
        self.line_ub4 = QFrame(self.PanelUB)
        self.line_ub4.setObjectName(u"line_ub4")
        self.line_ub4.setGeometry(QRect(310, 30, 3, 61))
        self.line_ub4.setFrameShape(QFrame.VLine)
        self.line_ub4.setFrameShadow(QFrame.Sunken)
        self.label_ub8 = QLabel(self.PanelUB)
        self.label_ub8.setObjectName(u"label_ub8")
        self.label_ub8.setGeometry(QRect(320, 50, 21, 17))
        self.line_ub5 = QFrame(self.PanelUB)
        self.line_ub5.setObjectName(u"line_ub5")
        self.line_ub5.setGeometry(QRect(350, 30, 3, 61))
        self.line_ub5.setFrameShape(QFrame.VLine)
        self.line_ub5.setFrameShadow(QFrame.Sunken)
        self.line_ub6 = QFrame(self.PanelUB)
        self.line_ub6.setObjectName(u"line_ub6")
        self.line_ub6.setGeometry(QRect(380, 30, 3, 61))
        self.line_ub6.setFrameShape(QFrame.VLine)
        self.line_ub6.setFrameShadow(QFrame.Sunken)
        self.txt22_ub.raise_()
        self.txt23_ub.raise_()
        self.txt21_ub.raise_()
        self.txt32_ub.raise_()
        self.txt33_ub.raise_()
        self.txt31_ub.raise_()
        self.txt11_ub.raise_()
        self.txt12_ub.raise_()
        self.txt13_ub.raise_()
        self.label_ub1.raise_()
        self.label_ub2.raise_()
        self.label_ub3.raise_()
        self.label_ub6.raise_()
        self.label_ub7.raise_()
        self.label_ub5.raise_()
        self.label_ub4.raise_()
        self.line_ub1.raise_()
        self.line_ub2.raise_()
        self.line_ub3.raise_()
        self.line_ub4.raise_()
        self.label_ub8.raise_()
        self.line_ub5.raise_()
        self.line_ub6.raise_()
        self.label_runnotag = QLabel(self.centralwidget)
        self.label_runnotag.setObjectName(u"label_runnotag")
        self.label_runnotag.setGeometry(QRect(130, 0, 67, 17))
        self.label_runno = QLabel(self.centralwidget)
        self.label_runno.setObjectName(u"label_runno")
        self.label_runno.setGeometry(QRect(200, 0, 231, 17))
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 594, 28))
        self.menuFile = QMenu(self.menubar)
        self.menuFile.setObjectName(u"menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.txt11_ub, self.txt12_ub)
        QWidget.setTabOrder(self.txt12_ub, self.txt13_ub)
        QWidget.setTabOrder(self.txt13_ub, self.txt21_ub)
        QWidget.setTabOrder(self.txt21_ub, self.txt22_ub)
        QWidget.setTabOrder(self.txt22_ub, self.txt23_ub)
        QWidget.setTabOrder(self.txt23_ub, self.txt31_ub)
        QWidget.setTabOrder(self.txt31_ub, self.txt32_ub)
        QWidget.setTabOrder(self.txt32_ub, self.txt33_ub)
        QWidget.setTabOrder(self.txt33_ub, self.txt_proj_ax11)
        QWidget.setTabOrder(self.txt_proj_ax11, self.txt_proj_ax12)
        QWidget.setTabOrder(self.txt_proj_ax12, self.txt_proj_ax13)
        QWidget.setTabOrder(self.txt_proj_ax13, self.txt_proj_ax21)
        QWidget.setTabOrder(self.txt_proj_ax21, self.txt_proj_ax22)
        QWidget.setTabOrder(self.txt_proj_ax22, self.txt_proj_ax23)
        QWidget.setTabOrder(self.txt_proj_ax23, self.txt_proj_ax31)
        QWidget.setTabOrder(self.txt_proj_ax31, self.txt_proj_ax32)
        QWidget.setTabOrder(self.txt_proj_ax32, self.txt_proj_ax33)
        QWidget.setTabOrder(self.txt_proj_ax33, self.txt_proj_t1)
        QWidget.setTabOrder(self.txt_proj_t1, self.txt_proj_u1)
        QWidget.setTabOrder(self.txt_proj_u1, self.txt_proj_t2)
        QWidget.setTabOrder(self.txt_proj_t2, self.txt_proj_u2)
        QWidget.setTabOrder(self.txt_proj_u2, self.txt_proj_t3)
        QWidget.setTabOrder(self.txt_proj_t3, self.txt_proj_u3)
        QWidget.setTabOrder(self.txt_proj_u3, self.btClr_proj)
        QWidget.setTabOrder(self.btClr_proj, self.btProj)
        QWidget.setTabOrder(self.btProj, self.btAutoFill)
        QWidget.setTabOrder(self.btAutoFill, self.txt_sl_ax11)
        QWidget.setTabOrder(self.txt_sl_ax11, self.txt_sl_ax12)
        QWidget.setTabOrder(self.txt_sl_ax12, self.btAutoFillAx1)
        QWidget.setTabOrder(self.btAutoFillAx1, self.cb_sl_ax1)
        QWidget.setTabOrder(self.cb_sl_ax1, self.txt_sl_ax13)
        QWidget.setTabOrder(self.txt_sl_ax13, self.txt_sl_ax21)
        QWidget.setTabOrder(self.txt_sl_ax21, self.txt_sl_ax22)
        QWidget.setTabOrder(self.txt_sl_ax22, self.btAutoFillAx2)
        QWidget.setTabOrder(self.btAutoFillAx2, self.cb_sl_ax2)
        QWidget.setTabOrder(self.cb_sl_ax2, self.txt_sl_ax23)
        QWidget.setTabOrder(self.txt_sl_ax23, self.txt_sl_ax31)
        QWidget.setTabOrder(self.txt_sl_ax31, self.txt_sl_ax32)
        QWidget.setTabOrder(self.txt_sl_ax32, self.btAutoFillAx3)
        QWidget.setTabOrder(self.btAutoFillAx3, self.cb_sl_ax3)
        QWidget.setTabOrder(self.cb_sl_ax3, self.txt_sl_ax33)
        QWidget.setTabOrder(self.txt_sl_ax33, self.btSlice)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menuFile.addAction(self.actionLoad)
        self.menuFile.addAction(self.actionLoad_Param)
        self.menuFile.addAction(self.actionSave_Param)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.actionLoad.setText(QCoreApplication.translate("MainWindow", u"Load ...", None))
        self.actionExit.setText(QCoreApplication.translate("MainWindow", u"Exit", None))
        self.actionLoad_Param.setText(QCoreApplication.translate("MainWindow", u"Load Param ...", None))
        self.actionSave_Param.setText(QCoreApplication.translate("MainWindow", u"Save Param ...", None))
        self.PanelProj.setTitle(QCoreApplication.translate("MainWindow", u"Projection", None))
        self.label_pr5.setText(QCoreApplication.translate("MainWindow", u"Q1:", None))
        self.label_pr6.setText(QCoreApplication.translate("MainWindow", u"Q2:", None))
        self.label_pr7.setText(QCoreApplication.translate("MainWindow", u"Q3:", None))
        self.label_pr1.setText(QCoreApplication.translate("MainWindow", u"H", None))
        self.label_pr2.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.label_pr3.setText(QCoreApplication.translate("MainWindow", u"L", None))
        self.label_pr4.setText(QCoreApplication.translate("MainWindow", u"Axis Label", None))
        self.btProj.setText(QCoreApplication.translate("MainWindow", u"Projection", None))
        self.btClr_proj.setText(QCoreApplication.translate("MainWindow", u"Clear", None))
        self.label_pr4_2.setText(QCoreApplication.translate("MainWindow", u"unit", None))
        self.PanelSlice.setTitle(QCoreApplication.translate("MainWindow", u"Slicing", None))
        self.label_sl4.setText(QCoreApplication.translate("MainWindow", u"Q1:", None))
        self.label_sl5.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.btAutoFillAx1.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.cb_sl_ax1.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.cb_sl_ax1.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.cb_sl_ax1.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))

        self.btAutoFillAx2.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.label_sl7.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.cb_sl_ax2.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.cb_sl_ax2.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.cb_sl_ax2.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))

        self.label_sl6.setText(QCoreApplication.translate("MainWindow", u"Q2:", None))
        self.btAutoFillAx3.setText(QCoreApplication.translate("MainWindow", u"A", None))
        self.label_sl9.setText(QCoreApplication.translate("MainWindow", u"to", None))
        self.cb_sl_ax3.setItemText(0, QCoreApplication.translate("MainWindow", u"X-Axis", None))
        self.cb_sl_ax3.setItemText(1, QCoreApplication.translate("MainWindow", u"Y-Axis", None))
        self.cb_sl_ax3.setItemText(2, QCoreApplication.translate("MainWindow", u"Thickness", None))

        self.label_sl8.setText(QCoreApplication.translate("MainWindow", u"Q3:", None))
        self.btAutoFill.setText(QCoreApplication.translate("MainWindow", u"Auto", None))
        self.label_sl1.setText(QCoreApplication.translate("MainWindow", u"Range", None))
        self.label_sl2.setText(QCoreApplication.translate("MainWindow", u"Plot Axis", None))
        self.label_sl3.setText(QCoreApplication.translate("MainWindow", u"Step", None))
        self.btSlice.setText(QCoreApplication.translate("MainWindow", u"Slice", None))
        self.PanelUB.setTitle(QCoreApplication.translate("MainWindow", u"UB matrix", None))
        self.label_ub1.setText(QCoreApplication.translate("MainWindow", u"Qx", None))
        self.label_ub2.setText(QCoreApplication.translate("MainWindow", u"Qy", None))
        self.label_ub3.setText(QCoreApplication.translate("MainWindow", u"Qz", None))
        self.label_ub6.setText(QCoreApplication.translate("MainWindow", u"K", None))
        self.label_ub7.setText(QCoreApplication.translate("MainWindow", u"L", None))
        self.label_ub5.setText(QCoreApplication.translate("MainWindow", u"H", None))
        self.label_ub4.setText(QCoreApplication.translate("MainWindow", u"=", None))
        self.label_ub8.setText(QCoreApplication.translate("MainWindow", u"X", None))
        self.label_runnotag.setText(QCoreApplication.translate("MainWindow", u"RunNo :", None))
        self.label_runno.setText(QCoreApplication.translate("MainWindow", u"None", None))
        self.menuFile.setTitle(QCoreApplication.translate("MainWindow", u"File", None))
    # retranslateUi




from __future__ import print_function
import os
import sys
import subprocess
import datetime
import pickle
ROUTE_GIT = 'https://hitorigochi:h1kar1t0kage@github.com/mlfcomp/'

GIT_TMP_PATH = 'git'

DIC_PATH = './.DIC_MAKEPACKAGE'
#INSTALL_SCRIPT = "Install_4_0_py3.py"
INSTALL_SCRIPT = "Install_4_0_py3_2004.py"
UTSUSEMI_RELEASE = "4.0"
#DISTVER = "Ubuntu_1604_1804LTS_PY3"
DISTVER = "Ubuntu_1604_2004LTS_PY3"

PACK_LIST = { 'manyo-core':("manyo","master"),
              'manyo-core-034':("manyo","0.3.4"),
              'utsusemi-core-037':("Utsusemi","0.3.7"),
              'utsusemi-core-40':("Utsusemi","4.0"),
              'ugao-master':("uGao","master"),
              'emaki-master':("emaki","master"),
              'utsusemi-ENG-master':("Utsusemi-ENG","master"),
              'utsusemi-ENG-40':("Utsusemi-ENG","4.0"),
              'utsusemi-TKtools-master':("Utsusemi-TKtools","master"),
              'utsusemi-SAS-master':("Utsusemi-SAS","master"),
              'utsusemi-SAS-40':("Utsusemi-SAS","4.0"),
              'utsusemi-DNA-master':("Utsusemi-DNA","master"),
              }

"""
RELEASE_LIST = {'4.0':['manyo-core','utsusemi-core-40','ugao-master','emaki-master','utsusemi-ENG-master','utsusemi-TKtools-master','utsusemi-SAS-master','utsusemi-DNA-master'],
                '0.3.7':['manyo-core','utsusemi-core-037','ugao-master','emaki-master','utsusemi-ENG-master','utsusemi-TKtools-master','utsusemi-SAS-master','utsusemi-DNA-master']
}n
"""
RELEASE_LIST = {'4.0':['manyo-core-034','utsusemi-core-40','ugao-master','emaki-master','utsusemi-ENG-40','utsusemi-TKtools-master','utsusemi-SAS-40','utsusemi-DNA-master'],
                '0.3.7':['manyo-core-034','utsusemi-core-037','ugao-master','emaki-master','utsusemi-ENG-master','utsusemi-TKtools-master','utsusemi-SAS-master']
}
def updateGitRepo( repo_name, branch ):
    print("--- updateGitRepo")
    com = "cd %s;git fetch %s"%(os.path.join(GIT_TMP_PATH,repo_name), os.path.join( ROUTE_GIT, repo_name ))
    os.system( com )
    com = "cd %s;git pull %s %s"%(os.path.join(GIT_TMP_PATH,repo_name), os.path.join( ROUTE_GIT, repo_name ), branch )
    os.system( com )

def getGitLastUpdate( repo_name, branch ):
    print("--- getGitLastUpdate")
    current_path = os.getcwd()
    os.chdir(os.path.join(GIT_TMP_PATH,repo_name))
    #v=str( subprocess.check_output("git log -b %s -n 1"%(branch), universal_newlines=True ) )
    #v=str( subprocess.check_output(["git","log","-b",branch,"-n""1"], universal_newlines=True ) )
    os.system( "git checkout %s"%(branch) )
    v=str( subprocess.check_output(["git","log","-n""1"], universal_newlines=True ) )
    #print( "v="+v )
    os.chdir(current_path)
    mon_dic = {"Jan":1,"Feb":2,"Mar":3,"Apr":4,"May":5,"Jun":6,"Jul":7,"Aug":8,"Sep":9,"Oct":10,"Nov":11,"Dec":12}
    s=v.split("\n")
    #print( "s=",s )

    date_info = ""
    for a_s in s:
        if a_s.find("Date:")==0:
            date_info = a_s
    if date_info=="":
        raise UserWarning(" Git version info is invalid." )

    ll=[]
    for k in date_info.split(" "):
        if k!="" and k!="Date:":
            if k in mon_dic.keys():
                ll.append( mon_dic[k] )
            else:
                ll.append( k )
                
    t=ll[3].split(':')
    #print( "ll=",ll )
    #print( "t=",t )
    p=datetime.datetime(int(ll[4]),int(ll[1]),int(ll[2]),int(t[0]),int(t[1]),int(t[2]))
    return p


def addDict( filepath, title, repo_name, branch ):
    print("--- addDict")
    rev_dic = getDict( filepath )
    #print("rev_dic after getDict=",rev_dic)
    add_dic = None
    p=getGitLastUpdate( repo_name, branch )
    add_dic=( repo_name, branch, p )
    rev_dic[title] = add_dic

    fo = open(filepath,"wb")
    pickle.dump( rev_dic,fo)
    fo.close()
        
            
def getDict( filepath ):
    print("--- getDict")
    rev_dic = {}
    if os.path.exists( filepath ):
        try:
            fo = open(filepath,"rb")
            rev_dic = pickle.load(fo)
            fo.close()
        except:
            print("False to read %s"%filepath)
    return rev_dic


def checkUpdate(filepath, title, repo_name, branch):
    print("--- checkUpdate")
    rev_dic = getDict( filepath )
    if (rev_dic==None):
        return (False,False)

    if title in rev_dic:
        info = rev_dic[title]
        if info[0]!=repo_name or info[1]!=branch:
            addDict( filepath, title, repo_name, branch )
            return (True,True)
        
        p_dic = info[2]
        p_git = getGitLastUpdate( repo_name, branch )
        if p_dic<p_git:
            return (True,True)
        else:
            return (True,False)
    else:
        return (False,False)


def makePackage(filepath,key_release):
    print("-- Make Package --")
    rev_dic = getDict(filepath)
    rq_pack_list = RELEASE_LIST[key_release]
    os.system( "mkdir -p tmp" )
    
    dt = datetime.datetime.now()
    date_s = "%04d%02d%02d"%(dt.year,dt.month,dt.day)
    date_s = date_s[2:]
    dir_name = "Utsusemi_%s_%s_%s"%(UTSUSEMI_RELEASE,date_s,DISTVER)
    dir_src = os.path.join( dir_name,"src" )
    os.system( "mkdir -p %s"%dir_src )
    
    manyo_core_rev = 0
    utsusemi_ver = ""
    sas_ver = ""
    dna_ver = ""
    
    coms = []
    for rq_pack in rq_pack_list:
        repo_name,branch=PACK_LIST[rq_pack]
        info = rev_dic[rq_pack]

        #coms.append( "cd tmp;git clone -b %s --depth 1 %s%s.git "%(branch,ROUTE_GIT,route) )
        
        if rq_pack.find("manyo-core")==0:
            coms.append( "cp -r %s/manyo %s/manyo-core"%(GIT_TMP_PATH,dir_src) )
            coms.append( "rm -rf %s/manyo-core/.git"%(dir_src) )
            manyo_core_rev = info[2].strftime('%Y%m%d')
        elif rq_pack.find("utsusemi-core")==0:
            coms.append( "cp -r %s/Utsusemi/manyo %s/manyo-utsusemi"%(GIT_TMP_PATH,dir_src) )
            coms.append( "rm -rf %s/manyo-utsusemi/.git"%(dir_src) )
            utsusemi_ver = info[2].strftime('%Y%m%d')
            #coms.append( 'cd %s/manyo-utsusemi;python ../../../script/insertRevNo.py "manyo" "r%s"'%(dir_src,utsusemi_ver) )
            coms.append( "mkdir -p %s/python-utsusemi"%(dir_src) )
            coms.append( "cp -r %s/Utsusemi/python-utsusemi %s/python-utsusemi/utsusemi"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi/python-utsusemi-SIK %s/python-utsusemi/utsusemi/SIK"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi/python-utsusemi-AMR %s/python-utsusemi/utsusemi/AMR"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi/python-utsusemi-HPN %s/python-utsusemi/utsusemi/HPN"%(GIT_TMP_PATH,dir_src) )
        elif rq_pack.find('ugao')==0:
            ugao_branch = info[1]
            ugao_ver = info[2].strftime('%Y%m%d')
            coms.append( "mkdir -p %s/python-ugao"%(dir_src) )
            coms.append( "cp -r %s/uGao %s/python-ugao/"%(GIT_TMP_PATH,dir_src) )
            coms.append( "rm -rf %s/python-ugao/uGao/.git"%(dir_src) )
        elif rq_pack.find('emaki')==0:
            coms.append( "cp -r %s/emaki %s/python-emaki"%(GIT_TMP_PATH,dir_src) )
            coms.append( "rm -rf %s/python-emaki/.git"%(dir_src) )
        elif rq_pack.find('utsusemi-ENG')==0:
            coms.append( "cp -r %s/Utsusemi-ENG/manyo %s/manyo-emaki"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi-ENG/python-utsusemi-ENG %s/python-utsusemi/utsusemi/ENG"%(GIT_TMP_PATH,dir_src) )
        elif rq_pack.find('utsusemi-TKtools')==0:
            coms.append( "cp -r %s/Utsusemi-TKtools/manyo %s/manyo-TKtools"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi-TKtools/python-utsusemi-TKtools %s/python-utsusemi/utsusemi/TKtools"%(GIT_TMP_PATH,dir_src) )
        elif rq_pack.find('utsusemi-SAS')==0:
            sas_branch = info[1]
            sas_ver = info[2].strftime('%Y%m%d')
            coms.append( "cp -r %s/Utsusemi-SAS/manyo %s/manyo-SAS"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi-SAS/python-utsusemi-SAS %s/python-utsusemi/utsusemi/SAS"%(GIT_TMP_PATH,dir_src) )
        elif rq_pack.find('utsusemi-DNA')==0:
            dna_branch = info[1]
            dna_ver = info[2].strftime('%Y%m%d')
            coms.append( "cp -r %s/Utsusemi-DNA/manyo %s/manyo-DNA"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi-DNA/lib-levmar %s/lib-levmar"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi-DNA/manyo-LevmarTools %s/manyo-LevmarTools"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi-DNA/python-utsusemi-DNA %s/python-utsusemi/utsusemi/DNA"%(GIT_TMP_PATH,dir_src) )
            coms.append( "cp -r %s/Utsusemi-DNA/environ-home %s/environ-DNA"%(GIT_TMP_PATH,dir_src) )
        else:
            pass

    coms.append( "cd %s;echo %s > version.txt"%( dir_src,("date:"+date_s) ) )
    coms.append( 'cd %s;echo "Manyo-lib rev=%s" >> version.txt' %( dir_src,manyo_core_rev ) )
    coms.append( 'cd %s;echo "uGao %s(%s)" >> version.txt' %( dir_src,ugao_branch,ugao_ver ) )
    coms.append( 'cd %s;echo "Utsusemi %s(%s)" >> version.txt' %( dir_src,UTSUSEMI_RELEASE,utsusemi_ver ) )
    if sas_ver!="":
        coms.append( 'cd %s;echo "SAS %s(%s)" >> version.txt' %( dir_src,sas_branch,sas_ver ) )
    if dna_ver!="":
        coms.append( 'cd %s;echo "DNA %s(%s)" >> version.txt' %( dir_src,dna_branch,dna_ver ) )
    coms.append( "cp script/%s %s/Install.py"%(INSTALL_SCRIPT,dir_name) )
    coms.append( "tar czvf %s.tar.gz %s"%(dir_name,dir_name) )
    #coms.append( "zip %s.zip -r %s"%(dir_name,dir_name) )
    coms.append( "rm -rf %s"%(dir_name) )
    coms.append( "rm -rf tmp/*" )
    
    for com in coms:
        print(com)
        if os.system(com)!=0:
            print( "!!! ERROR !!!" )
            print( "command = %s"%com )
            isGoodEnd = False
            break
        else:
            isGoodEnd = True
    
    return (isGoodEnd,dir_name)

def makePage(fname):
    print("-- Make Page --")
    
def showUsage():
    print( "Usage : python3 %s <release tag> [<options>]"%(sys.argv[0]) )
    print( "   " )
    print( "      <release tag> : 4.0/0.3.7 " )
    print( "      <options>" )
    print( "            --force : make package by force" )
    print( "                      If not, make pakage when any updated source codes" )
    print( "            --new   : not update but make new from source" )

if __name__ == '__main__':
    if sys.version.split(" ")[0]<"3.0.0":
        print( "ERROR : This script requires Python 3.x")
        print( "        Your python version is %s"%(sys.version.split(" ")[0]))
        sys.exit(-1)

    if len(sys.argv)==1:
        showUsage()
        sys.exit(1)
    
    isRequiredUpdate=False
    isNew = False
    key_release = ""
    print( sys.argv )
    for an_arg in sys.argv:
        if an_arg.find("--")==0:
            if an_arg=="--force":
                isRequiredUpdate=True
            if an_arg=="--new":
                isNew = True
        elif an_arg in RELEASE_LIST.keys():
            key_release = an_arg

    flag=False
    if key_release=="":
        showUsage()
        sys.exit(1)
    else:
        print( "key_release="+key_release )
        if isNew:
            print("--new")
            for title in RELEASE_LIST[key_release]:
                if title in PACK_LIST.keys():
                    repo_name,branch = PACK_LIST[title]
                    print( "repo_name,branch = %s, %s"%(repo_name,branch))
                    addDict( DIC_PATH, title, repo_name, branch )
        
        elif isRequiredUpdate:
            for title in RELEASE_LIST[key_release]:
                if title in PACK_LIST.keys():
                    repo_name,branch = PACK_LIST[title]
                    print( "repo_name="+repo_name+","+branch)
                    updateGitRepo( repo_name, branch )
                    addDict( DIC_PATH, title, repo_name, branch )
            flag,fname=makePackage(DIC_PATH,key_release)

        else:            
            for title in RELEASE_LIST[key_release]:
                if title in PACK_LIST.keys():
                    repo_name,branch = PACK_LIST[title]
                    updateGitRepo( repo_name, branch )
                    status, isUpdate = checkUpdate(DIC_PATH, title, repo_name, branch)
                    if status and isUpdate:
                        addDict( DIC_PATH, title, repo_name, branch )
                        isRequiredUpdate = True
                else:
                    print("title(%s) is invalid."%(title))
                    sys.exit(1)
                
                #if isRequiredUpdate:
                #    break

            if isRequiredUpdate:
                flag,fname=makePackage(DIC_PATH,key_release)

    if flag:
        makePage(fname)




from __future__ import print_function
import Manyo as mm
import math
#ttt
def TimeFocusDNA(eca, T0Shift, sd, ed, dbin, FastChopperUse = True):
    
    LSample = 42.0
    LFC = 7.725

    if (FastChopperUse):
        L1 = LSample-LFC
    else:
        L1 = LSample
    
    toflist = []
    ylist = []
    elist = []
    cntnumlist = []

    toflist.append(sd)
    while 1:
        toflist.append(toflist[-1]+dbin)
        ylist.append(0)
        elist.append(0)
        cntnumlist.append(0)
        
        if  toflist[-1] >= ed:
            break

    TOFbinNum = len(toflist)-1
    
    ecanew = mm.ElementContainerArray()
    ecanew.InputHeader(eca.PutHeaderPointer())

    for ii in range(eca.PutTableSize()):
        if ii % 100 ==0:
            print("%d / %d" %(ii,eca.PutTableSize()))
        
        ec = eca.Put(ii)
        ecnew = mm.ElementContainer()
        ecnew.InputHeader(ec.PutHeaderPointer())
        
        temptof = ec.PutXList()
        tempy = ec.PutYList()
        tempe = ec.PutEList()
        invL12 = 1.0/(GetL2(ec)+L1)
        pa = invL12/math.sin(GetPolerAngle(ec)*0.5)
        d = 0.0
        for jj in range(len(ylist)):
            ylist[jj] = 0.0
            elist[jj] = 0.0

        for jj in range(len(tempy)):
            d = 1.97801e-3*pa*((temptof[jj]+temptof[jj+1])/2-T0Shift)
            pp = int((d-sd)//dbin)
            """ 
            if ii == 325:
                print pp, len(ylist),(1.97801e-3*pa*((temptof[jj]+temptof[jj+1])/2-T0Shift)),d
            """
            if pp >=0 and pp <TOFbinNum:
                ylist[pp] += tempy[jj]
                elist[pp] = math.sqrt(elist[pp]**2+tempe[jj]**2)
            #else:
                #print pp, ii, jj

        ecnew.Add("TOF",toflist)
        ecnew.Add("Intensity", ylist)
        ecnew.Add("Error",elist)
        ecnew.SetKeys("TOF","Intensity","Error")

        ecanew.Add(ecnew)

    return ecanew

def GetL2(ec):
    pixpos = ec.PutHeader().PutDoubleVector("PixelPosition")
    #print len(pixpos)
    L2 = 0.0
    for ii in range(len(pixpos)):
        L2 += pixpos[ii]**2

    return math.sqrt(L2)/1000

def GetPolerAngle(ec):
    pixpa =  ec.PutHeader().PutDoubleVector("PixelPolarAngle")
    return pixpa[0]*math.pi/180.0



from __future__ import print_function
import utsusemi.ana.Reduction.EditWiringFile as ED
import math
import os

class EditWiringInfoForGUI:
    
    def __init__(self,filename,type_, pmin, pmax, pbin, Lambda,LambdaWidth):

        self.ed = ED.EditWiringInfo(filename)
        self.Type = type_
        self.params = []
        self.params.append(pmin)
        self.params.append(pmax)
        self.params.append(pbin)
        self.Lambda = Lambda
        self.LambdaWidth = LambdaWidth
        self.L12 = 44.1 #L1 = 42m, L2 = 2.1m

    #dfile = "/home/dnaadmin/dev/Diffraction/FB/DetectorInfoTYMM20140120.xml"
    
    def SetTofBinPattern(self):
        if(self.Type == 22):
            self.params.append(0.0)
            self.params.append(0.0)
            self.params.append(1.0)

        self.ed.change_TofBinPattern(0,self.Type,self.params)
    
    def SetFrameInfo(self):
        Lambdamin = self.Lambda-self.LambdaWidth*0.5
        Tofmin = 252.8*Lambdamin*self.L12

        Lambdamax = self.Lambda+self.LambdaWidth*0.5
        Tofmax = 252.8*Lambdamax*self.L12
        
        if math.ceil(Tofmin/40000) == math.ceil(Tofmax/40000) :
            boundary = 0.0
        else:
            boundary = math.fmod(Tofmin, 40000)

        frameNo = math.ceil(Tofmax / 40000)
        #boundary = Tofmin % 40000.0

        print("DiffractionGUIMain >>", Tofmin, Tofmax, frameNo, boundary)

        self.ed.set_FrameInfo(frameNo,boundary)

    def OutputXml(self):
        self.SetTofBinPattern()
        self.SetFrameInfo()

        NewWIfilename = "WiringInfoGUItemp.xml"
        self.ed.SaveToXml(NewWIfilename)

        return os.path.join(os.getcwd(),NewWIfilename)
        





# -*- coding: utf-8 -*-

import Manyo
import PlaneSlicer
import uGao.M2PlotPlusCui as mp
import sys


def main():
    argv = sys.argv
    argc = len(argv)

    orgX = float(argv[1])
    orgY = float(argv[2])
    orgZ = float(argv[3])
    org  = [orgX, orgY, orgZ]

    x0   = float(argv[4])
    x1   = float(argv[5])
    x2   = float(argv[6])
    x    = [x0, x1, x2]

    y0   = float(argv[7])
    y1   = float(argv[8])
    y2   = float(argv[9])
    y    = [y0, y1, y2]

    xbin = float(argv[10])
    ybin = float(argv[11])

    rangeMin = float(argv[12])
    rangeMax = float(argv[13])
    rngx = [rangeMin, rangeMax]

    rangeMin = float(argv[14])
    rangeMax = float(argv[15])
    rngy = [rangeMin, rangeMax]

    thick = float(argv[16])






if __name__ == '__main__':
    main()



# The four rules of arithmetic, +, -, * and /, are prepared.
# These operators can be applied to
# data containers of Manyo Library, ElementContainer, ElementContainerArray
# and ElementContainerMatrix.

import Manyo.MLF

e = Manyo.ElementContainer()
e.Add("x",[0,1,2,3,4,5,6,7,8,9])
e.Add("y",[5,2,7,8,2,1,3,4,7])
e.Add("err",[1,1,1,1,1,1,1,1,1])

e.SetKeys("x","y","err")
# The length of the vector specified by the first argument
# should be one larger than those by the second and third arguments.

ep = e+e
em = ep-e
e2 = e*e
es = ep/e

# The following line(s) should be executed on
# python command line prompt with copy-and-paste method.
# The data viewer windows will be closed immediately.
ConView=Manyo.MLF.ContainerViewer()
ConView.Histogram(e)
ConView.Histogram(e2)



